@@ -1,30 +1,30 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of MySQL Connector/ODBC, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of MySQL Connector/ODBC, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /**
   @file  desc.c
@@ -39,6 +39,7 @@
  ****************************************************************************/
 
 #include "driver.h"
+#include <algorithm>
 
 /* Utility macros for defining descriptor fields */
 #define HDR_FLD(field, perm, type) \
@@ -46,6 +47,7 @@
     {(perm), (type), DESC_HDR, offsetof(DESC, field)}
     /* parens around field in offsetof() confuse GCC */
 
+
 #define REC_FLD(field, perm, type) \
   static desc_field REC_##field= \
     {(perm), (type), DESC_REC, offsetof(DESCREC, field)}
@@ -59,40 +61,10 @@
 DESC *desc_alloc(STMT *stmt, SQLSMALLINT alloc_type,
                  desc_ref_type ref_type, desc_desc_type desc_type)
 {
-  DESC *desc= (DESC *)myodbc_malloc(sizeof(DESC), MYF(MY_ZEROFILL));
+  DESC *desc= new DESC(stmt, alloc_type, ref_type, desc_type);
   if (!desc)
     return NULL;
-  /*
-     We let the dynamic array handle the memory for the whole DESCREC,
-     but in desc_get_rec we manually get a pointer to it. This avoids
-     having to call set_dynamic after modifying the DESCREC.
-  */
-  if (myodbc_init_dynamic_array(&desc->records, sizeof(DESCREC), 0, 0))
-  {
-    x_free((char *)desc);
-    return NULL;
-  }
 
-  if (myodbc_init_dynamic_array(&desc->bookmark, sizeof(DESCREC), 0, 0))
-  {
-    delete_dynamic(&desc->records);
-    x_free((char *)desc);
-    return NULL;
-  }
-
-  desc->desc_type= desc_type;
-  desc->alloc_type= alloc_type;
-  desc->ref_type= ref_type;
-  desc->stmt= stmt;
-  /* spec-defined defaults/initialization */
-  desc->array_size= 1;
-  desc->array_status_ptr= NULL;
-  desc->bind_offset_ptr= NULL;
-  desc->bind_type= SQL_BIND_BY_COLUMN;
-  desc->count= 0;
-  desc->bookmark_count= 0;
-  desc->rows_processed_ptr= NULL;
-  desc->exp.stmts= NULL;
   return desc;
 }
 
@@ -102,109 +74,115 @@ DESC *desc_alloc(STMT *stmt, SQLSMALLINT alloc_type,
 */
 void desc_free(DESC *desc)
 {
-  assert(desc);
-  if (IS_APD(desc))
-    desc_free_paramdata(desc);
-  delete_dynamic(&desc->records);
-  delete_dynamic(&desc->bookmark);
-  x_free(desc);
+  delete desc;
 }
 
 
-/*
-  Free any memory allocated for SQLPutData(). This is only useful
-  for APDs.
-*/
-void desc_free_paramdata(DESC *desc)
+void DESC::reset()
 {
-  SQLLEN i;
-  for (i= 0; i < desc->count; ++i)
-  {
-    DESCREC *aprec= desc_get_rec(desc, i, FALSE);
-    assert(aprec);
-    if (aprec->par.alloced)
-    {
-      aprec->par.alloced= FALSE;
-      x_free(aprec->par.value);
-    }
-  }
+  records2.clear();
+}
+
+void DESC::free_paramdata()
+{
+  for (auto &r : records2)
+    r.par.reset();
 }
 
 
 /*
  * Initialize APD
  */
-void desc_rec_init_apd(DESCREC *rec)
+void DESCREC::desc_rec_init_apd()
 {
-  memset(rec, 0, sizeof(DESCREC));
   /* ODBC defaults */
-  rec->concise_type= SQL_C_DEFAULT;
-  rec->data_ptr= NULL;
-  rec->indicator_ptr= NULL;
-  rec->octet_length_ptr= NULL;
-  rec->type= SQL_C_DEFAULT;
+  concise_type= SQL_C_DEFAULT;
+  data_ptr= NULL;
+  indicator_ptr= NULL;
+  octet_length_ptr= NULL;
+  type= SQL_C_DEFAULT;
 
   /* internal */
-  rec->par.alloced= FALSE;
-  rec->par.value= NULL;
+  par.reset();
 }
 
 
 /*
  * Initialize IPD
  */
-void desc_rec_init_ipd(DESCREC *rec)
+void DESCREC::desc_rec_init_ipd()
 {
-  memset(rec, 0, sizeof(DESCREC));
   /* ODBC defaults */
-  rec->fixed_prec_scale= SQL_TRUE;
-  rec->local_type_name= (SQLCHAR *)"";
-  rec->nullable= SQL_NULLABLE;
-  rec->parameter_type= SQL_PARAM_INPUT;
-  rec->type_name= (SQLCHAR *)"VARCHAR";
-  rec->is_unsigned= SQL_FALSE;
+  fixed_prec_scale= SQL_TRUE;
+  local_type_name= (SQLCHAR *)"";
+  nullable= SQL_NULLABLE;
+  parameter_type= SQL_PARAM_INPUT;
+  type_name= (SQLCHAR *)"VARCHAR";
+  is_unsigned= SQL_FALSE;
 
   /* driver defaults */
-  rec->name= (SQLCHAR *)"";
+  name= (SQLCHAR *)"";
 }
 
 
 /*
  * Initialize ARD
  */
-void desc_rec_init_ard(DESCREC *rec)
+void DESCREC::desc_rec_init_ard()
 {
-  memset(rec, 0, sizeof(DESCREC));
   /* ODBC defaults */
-  rec->concise_type= SQL_C_DEFAULT;
-  rec->data_ptr= NULL;
-  rec->indicator_ptr= NULL;
-  rec->octet_length_ptr= NULL;
-  rec->type= SQL_C_DEFAULT;
+  concise_type= SQL_C_DEFAULT;
+  data_ptr= NULL;
+  indicator_ptr= NULL;
+  octet_length_ptr= NULL;
+  type= SQL_C_DEFAULT;
 }
 
 
 /*
  * Initialize IRD
  */
-void desc_rec_init_ird(DESCREC *rec)
+void DESCREC::desc_rec_init_ird()
 {
-  memset(rec, 0, sizeof(DESCREC));
   /* ODBC defaults */
   /* driver defaults */
-  rec->auto_unique_value= SQL_FALSE;
-  rec->case_sensitive= SQL_TRUE;
-  rec->concise_type= SQL_VARCHAR;
-  rec->display_size= 100;/*?*/
-  rec->fixed_prec_scale= SQL_TRUE;
-  rec->length= 100;/*?*/
-  rec->nullable= SQL_NULLABLE_UNKNOWN;
-  rec->type= SQL_VARCHAR;
-  rec->type_name= (SQLCHAR *)"VARCHAR";
-  rec->unnamed= SQL_UNNAMED;
-  rec->is_unsigned= SQL_FALSE;
+  auto_unique_value= SQL_FALSE;
+  case_sensitive= SQL_TRUE;
+  concise_type= SQL_VARCHAR;
+  display_size= 100;/*?*/
+  fixed_prec_scale= SQL_TRUE;
+  length= 100;/*?*/
+  nullable= SQL_NULLABLE_UNKNOWN;
+  type= SQL_VARCHAR;
+  type_name= (SQLCHAR *)"VARCHAR";
+  unnamed= SQL_UNNAMED;
+  is_unsigned= SQL_FALSE;
 }
 
+void DESCREC::reset_to_defaults()
+{
+  par.reset();
+  row.reset();
+
+  if (m_desc_type == DESC_PARAM && m_ref_type == DESC_APP)
+    desc_rec_init_apd();
+  else if (m_desc_type == DESC_PARAM && m_ref_type == DESC_IMP)
+    desc_rec_init_ipd();
+  else if (m_desc_type == DESC_ROW && m_ref_type == DESC_APP)
+    desc_rec_init_ard();
+  else if (m_desc_type == DESC_ROW && m_ref_type == DESC_IMP)
+    desc_rec_init_ird();
+
+}
+
+void DESCREC::par_struct::add_param_data(const char *chunk,
+                                         unsigned long length)
+{
+    tempbuf.add_to_buffer(chunk, length);
+
+    // TODO: do the memory errors
+    //  return stmt->set_error(MYERR_S1001, NULL, 4001);
+}
 
 /*
  * Get a record from the descriptor.
@@ -227,30 +205,18 @@ DESCREC *desc_get_rec(DESC *desc, int recnum, my_bool expand)
     {
       if (!desc->bookmark_count)
       {
-        rec= (DESCREC *)alloc_dynamic(&desc->bookmark);
-        if (!rec)
-          return NULL;
+        desc->bookmark2.emplace_back(desc->desc_type, desc->ref_type);
+        rec = &desc->bookmark2.back();
 
-        memset(rec, 0, sizeof(DESCREC));
         ++desc->bookmark_count;
-
-        /* record initialization */
-        if (IS_APD(desc))
-            desc_rec_init_apd(rec);
-        else if (IS_IPD(desc))
-            desc_rec_init_ipd(rec);
-        else if (IS_ARD(desc))
-            desc_rec_init_ard(rec);
-        else if (IS_IRD(desc))
-            desc_rec_init_ird(rec);
       }
     }
 
-    rec= (DESCREC *)desc->bookmark.buffer;
+    rec= &desc->bookmark2.back();
   }
   else if (recnum < 0)
   {
-    set_stmt_error(desc->stmt, "07009", "Invalid descriptor index", MYERR_07009);
+    desc->stmt->set_error("07009", "Invalid descriptor index", MYERR_07009);
     return NULL;
   }
   else
@@ -259,37 +225,15 @@ DESCREC *desc_get_rec(DESC *desc, int recnum, my_bool expand)
     /* expand if needed */
     if (expand)
     {
-      for (i= desc->count; expand && i <= recnum; ++i)
+      for (i= desc->rcount(); expand && i <= recnum; ++i)
       {
-        /* we might have used records lying around from before if
-         * SQLFreeStmt() was called with SQL_UNBIND or SQL_FREE_PARAMS
-         */
-        if ((uint)i < desc->records.elements)
-        {
-          rec= ((DESCREC *)desc->records.buffer) + recnum;
-        }
-        else
-        {
-          rec= (DESCREC *)alloc_dynamic(&desc->records);
-          if (!rec)
-            return NULL;
-        }
-        memset(rec, 0, sizeof(DESCREC));
-        ++desc->count;
-
-        /* record initialization */
-        if (IS_APD(desc))
-            desc_rec_init_apd(rec);
-        else if (IS_IPD(desc))
-            desc_rec_init_ipd(rec);
-        else if (IS_ARD(desc))
-            desc_rec_init_ard(rec);
-        else if (IS_IRD(desc))
-            desc_rec_init_ird(rec);
+        desc->records2.emplace_back(desc->desc_type, desc->ref_type);
+        rec = &desc->records2.back();
+        rec->reset_to_defaults();
       }
     }
-    if (recnum < desc->count)
-      rec= ((DESCREC *)desc->records.buffer) + recnum;
+    if (recnum < desc->rcount())
+      rec = &desc->records2[recnum];
   }
 
   if (expand)
@@ -298,38 +242,6 @@ DESCREC *desc_get_rec(DESC *desc, int recnum, my_bool expand)
 }
 
 
-/*
- * Disassociate a statement from an explicitly allocated
- * descriptor.
- *
- * @param desc The descriptor
- * @param stmt The statement
- */
-void desc_remove_stmt(DESC *desc, STMT *stmt)
-{
-  LIST *lstmt;
-
-  if (desc->alloc_type != SQL_DESC_ALLOC_USER)
-    return;
-
-  for (lstmt= desc->exp.stmts; lstmt; lstmt= lstmt->next)
-  {
-    if (lstmt->data == stmt)
-    {
-      desc->exp.stmts= list_delete(desc->exp.stmts, lstmt);
-      /* Free only if it was the last element */
-      //if(!lstmt->next && !lstmt->prev)
-      {
-        x_free(lstmt);
-      }
-      return;
-    }
-  }
-
-  assert(!"Statement was not associated with descriptor");
-}
-
-
 /*
  * Check with the given descriptor contains any data-at-exec
  * records. Return the record number or -1 if none are found.
@@ -339,7 +251,7 @@ int desc_find_dae_rec(DESC *desc)
   int i;
   DESCREC *rec;
   SQLLEN *octet_length_ptr;
-  for (i= 0; i < desc->count; ++i)
+  for (i= 0; i < desc->rcount(); ++i)
   {
     rec= desc_get_rec(desc, i, FALSE);
     assert(rec);
@@ -369,7 +281,7 @@ DESCREC * desc_find_outstream_rec(STMT *stmt, uint *recnum, uint *res_col_num)
 
 /* No streams in iODBC */
 #ifndef USE_IODBC
-  for (i= start; i < stmt->ipd->count; ++i)
+  for (i= start; i < stmt->ipd->rcount(); ++i)
   {
     rec= desc_get_rec(stmt->ipd, i, FALSE);
     assert(rec);
@@ -444,6 +356,7 @@ apply_desc_val(void *dest, SQLSMALLINT dest_type, void *src, SQLINTEGER buflen)
       *(SQLULEN *)dest= (SQLULEN)src;
     break;
 
+  case SQL_NTS:
   case SQL_IS_POINTER:
     *(SQLPOINTER *)dest= src;
     break;
@@ -493,7 +406,7 @@ getfield(SQLSMALLINT fldid)
   REC_FLD(literal_prefix   , PR_RIR                     , SQL_IS_POINTER);
   REC_FLD(literal_suffix   , PR_RIR                     , SQL_IS_POINTER);
   REC_FLD(local_type_name  , PR_RIR|PR_RIP              , SQL_IS_POINTER);
-  REC_FLD(name             , PR_RIR|PR_WIP              , SQL_IS_POINTER);
+  REC_FLD(name             , PR_RIR|PR_WIP              , SQL_NTS);
   REC_FLD(nullable         , PR_RIR|PR_RIP              , SQL_IS_SMALLINT);
   REC_FLD(num_prec_radix   , PR_WAR|PR_WAP|PR_RIR|PR_WIP, SQL_IS_INTEGER);
   REC_FLD(octet_length     , PR_WAR|PR_WAP|PR_RIR|PR_WIP, SQL_IS_LEN);
@@ -612,8 +525,8 @@ SQLRETURN
 MySQLGetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
                   SQLPOINTER valptr, SQLINTEGER buflen, SQLINTEGER *outlen)
 {
-  desc_field *fld= getfield(fldid);
   DESC *desc= (DESC *)hdesc;
+  desc_field *fld = getfield(fldid);
   void *src_struct;
   void *src;
 
@@ -666,7 +579,7 @@ MySQLGetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
     src_struct= desc;
   else
   {
-    if (recnum < 1 || recnum > desc->count)
+    if (recnum < 1 || recnum > desc->rcount())
       return set_desc_error(desc, "07009",
                 "Invalid descriptor index",
                 MYERR_07009);
@@ -674,6 +587,15 @@ MySQLGetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
     assert(src_struct);
   }
 
+  if (fldid == SQL_DESC_COUNT)
+  {
+    /*
+      This requires special treatment because the field is dynamically
+      obtained from the function
+    */
+    desc->rcount();
+  }
+
   src= ((char *)src_struct) + fld->offset;
 
   /* TODO checks when strings? */
@@ -789,82 +711,98 @@ MySQLGetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
 }
 
 
+SQLRETURN DESC::set_error(char *state, const char *message, uint errcode)
+{
+  myodbc_stpmov(error.sqlstate, state);
+  strxmov(error.message, stmt->dbc->st_error_prefix,
+          message, NullS);
+  error.native_error = errcode;
+
+  return SQL_ERROR;
+}
+
+DESC::DESC(STMT *p_stmt, SQLSMALLINT p_alloc_type,
+    desc_ref_type p_ref_type, desc_desc_type p_desc_type) :
+    alloc_type(p_alloc_type), array_size(1), array_status_ptr(nullptr),
+    bind_offset_ptr(nullptr), bind_type(SQL_BIND_BY_COLUMN),
+    count(0), bookmark_count(0),
+    rows_processed_ptr(nullptr), desc_type(p_desc_type), ref_type(p_ref_type),
+    stmt(p_stmt), dbc(nullptr)
+{
+  if(stmt != nullptr)
+    dbc = p_stmt->dbc;
+}
+
+
 /*
   @type    : ODBC 3.0 API
   @purpose : Set a field of a descriptor.
  */
-SQLRETURN
-MySQLSetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
-                  SQLPOINTER val, SQLINTEGER buflen)
+SQLRETURN DESC::set_field(SQLSMALLINT recnum, SQLSMALLINT fldid,
+                      SQLPOINTER val, SQLINTEGER buflen)
 {
-  desc_field *fld= getfield(fldid);
-  DESC *desc= (DESC *)hdesc;
+  desc_field *fld = getfield(fldid);
   void *dest_struct;
   void *dest;
 
-  if (desc == NULL)
-  {
-    return SQL_INVALID_HANDLE;
-  }
-
-  CLEAR_DESC_ERROR(desc);
+  error.clear();
 
   /* check for invalid IRD modification */
-  if (IS_IRD(desc))
+  if (is_ird())
   {
     switch (fldid)
     {
-    case SQL_DESC_ARRAY_STATUS_PTR:
-    case SQL_DESC_ROWS_PROCESSED_PTR:
-      break;
-    default:
-      return set_desc_error(desc, "HY016",
-                            "Cannot modify an implementation row descriptor",
-                            MYERR_S1016);
+      case SQL_DESC_ARRAY_STATUS_PTR:
+      case SQL_DESC_ROWS_PROCESSED_PTR:
+        break;
+      default:
+      return set_error("HY016",
+                      "Cannot modify an implementation row descriptor",
+                      MYERR_S1016);
     }
   }
 
   if ((fld == NULL) ||
       /* header permissions check */
       (fld->loc == DESC_HDR &&
-         ((desc->ref_type == DESC_APP && (~fld->perms & P_WA)) ||
-          (desc->ref_type == DESC_IMP && (~fld->perms & P_WI)))))
+         ((ref_type == DESC_APP && (~fld->perms & P_WA)) ||
+          (ref_type == DESC_IMP && (~fld->perms & P_WI)))))
   {
-    return set_desc_error(desc, "HY091",
-              "Invalid descriptor field identifier",
-              MYERR_S1091);
+    return set_error("HY091",
+                     "Invalid descriptor field identifier",
+                     MYERR_S1091);
   }
   else if (fld->loc == DESC_REC)
   {
     int perms= 0; /* needed perms to access */
 
-    if (desc->ref_type == DESC_APP)
+    if (ref_type == DESC_APP)
       perms= P_WA;
-    else if (desc->ref_type == DESC_IMP)
+    else if (ref_type == DESC_IMP)
       perms= P_WI;
 
-    if (desc->desc_type == DESC_PARAM)
+    if (desc_type == DESC_PARAM)
       perms= P_PAR(perms);
-    else if (desc->desc_type == DESC_ROW)
+    else if (desc_type == DESC_ROW)
       perms= P_ROW(perms);
 
     if ((~fld->perms & perms) == perms)
-      return set_desc_error(desc, "HY091",
-                "Invalid descriptor field identifier",
-                MYERR_S1091);
+      return set_error("HY091",
+                       "Invalid descriptor field identifier",
+                       MYERR_S1091);
   }
 
   /* get the dest struct */
   if (fld->loc == DESC_HDR)
-    dest_struct= desc;
+    dest_struct= this;
   else
   {
-    if (recnum < 1 && desc->stmt->stmt_options.bookmarks == SQL_UB_OFF)
-      return set_desc_error(desc, "07009",
-                "Invalid descriptor index",
-                MYERR_07009);
+    if (recnum < 1 && stmt->stmt_options.bookmarks == SQL_UB_OFF)
+      return set_error("07009",
+                       "Invalid descriptor index",
+                       MYERR_07009);
     else
-      dest_struct= desc_get_rec(desc, recnum - 1, TRUE);
+      dest_struct= desc_get_rec(this, recnum - 1, TRUE);
   }
 
   dest= ((char *)dest_struct) + fld->offset;
@@ -876,27 +814,36 @@ MySQLSetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
   /* TODO checks when strings? */
   if ((fld->data_type == SQL_IS_POINTER && buflen != SQL_IS_POINTER) ||
       (fld->data_type != SQL_IS_POINTER && buflen == SQL_IS_POINTER))
-    return set_desc_error(desc, "HY015",
-                          "Invalid parameter type",
-                          MYERR_S1015);
+    return set_error("HY015",
+                     "Invalid parameter type",
+                     MYERR_S1015);
 
   /* per-field checks/functionality */
   switch (fldid)
   {
   case SQL_DESC_COUNT:
     /* we just force the descriptor record count to expand */
-    (void)desc_get_rec(desc, (size_t)val - 1, TRUE);
+    (void)desc_get_rec(this, (size_t)val - 1, TRUE);
     break;
   case SQL_DESC_NAME:
+    {
+      // dest_struct already points to the correct DESCREC*
+      DESCREC *name_rec = (DESCREC*)dest_struct;
+      // Add name as parameter data and zero terminating character
+      name_rec->par.add_param_data((char*)val, strlen((char*)val) + 1);
+      // Get a pointer to allocated copy of the name
+      val = name_rec->par.val();
+    }
+    break;
     /* We don't support named parameters, values stay as initialized */
-    return set_desc_error(desc, "01S01",
-                          "Option value changed",
-                          MYERR_01S02);
+    //return set_desc_error(desc, "01S01",
+    //                      "Option value changed",
+    //                      MYERR_01S02);
   case SQL_DESC_UNNAMED:
     if ((size_t)val == SQL_NAMED)
-      return set_desc_error(desc, "HY092",
-                            "Invalid attribute/option identifier",
-                            MYERR_S1092);
+      return set_error("HY092",
+                       "Invalid attribute/option identifier",
+                       MYERR_S1092);
   }
 
   /* We have to unbind the value if not setting a buffer */
@@ -921,7 +868,7 @@ MySQLSetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
     "ParameterType Argument" sectiosn - basically IPD has to be heres as well with same rules
     C and SQL types match. Thus we can use same function for calculation of type and dti code.
    */
-  if ((IS_ARD(desc) || IS_APD(desc) || IS_IPD(desc)) && fld->loc == DESC_REC)
+  if ((is_ard() || is_apd() || is_ipd()) && fld->loc == DESC_REC)
   {
     DESCREC *rec= (DESCREC *) dest_struct;
     switch (fldid)
@@ -951,7 +898,7 @@ MySQLSetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
     case SQL_DESC_TYPE:
     case SQL_DESC_CONCISE_TYPE:
       /* setup type specific defaults (TODO others besides SQL_C_NUMERIC)? */
-      if (IS_ARD(desc) && rec->type == SQL_C_NUMERIC)
+      if (is_ard() && rec->type == SQL_C_NUMERIC)
       {
         rec->precision= 38;
         rec->scale= 0;
@@ -963,7 +910,7 @@ MySQLSetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
     Set "real_param_done" for parameters if all fields needed to bind
     a parameter are set.
   */
-  if (IS_APD(desc) && val != NULL && fld->loc == DESC_REC)
+  if (is_apd() && val != NULL && fld->loc == DESC_REC)
   {
     DESCREC *rec= (DESCREC *) dest_struct;
     switch (fldid)
@@ -1002,27 +949,8 @@ SQLRETURN MySQLCopyDesc(SQLHDESC SourceDescHandle, SQLHDESC TargetDescHandle)
               "Associated statement is not prepared",
               MYERR_S1007);
 
-  /* copy the records */
-  delete_dynamic(&dest->records);
-  if (myodbc_init_dynamic_array(&dest->records, sizeof(DESCREC),
-                            src->records.max_element,
-                            src->records.alloc_increment))
-  {
-    return set_desc_error(dest, "HY001",
-              "Memory allocation error",
-              MYERR_S1001);
-  }
-  memcpy(dest->records.buffer, src->records.buffer,
-         src->records.max_element * src->records.size_of_element);
-
-  /* copy all fields */
-  dest->array_size= src->array_size;
-  dest->array_status_ptr= src->array_status_ptr;
-  dest->bind_offset_ptr= src->bind_offset_ptr;
-  dest->bind_type= src->bind_type;
-  dest->count= src->count;
-  dest->rows_processed_ptr= src->rows_processed_ptr;
-  memcpy(&dest->error, &src->error, sizeof(MYERROR));
+  /* copy the records, copy constructors should take care of everything */
+  *dest = *src;
 
   /* TODO consistency check on target, if needed (apd) */
 
@@ -1056,8 +984,9 @@ stmt_SQLSetDescField(STMT *stmt, DESC *desc, SQLSMALLINT recnum,
                      SQLSMALLINT fldid, SQLPOINTER val, SQLINTEGER buflen)
 {
   SQLRETURN rc;
-  if ((rc= MySQLSetDescField((SQLHANDLE)desc, recnum, fldid,
-                             val, buflen)) != SQL_SUCCESS)
+  CHECK_HANDLE(desc);
+
+  if ((rc= desc->set_field(recnum, fldid, val, buflen)) != SQL_SUCCESS)
     memcpy(&stmt->error, &desc->error, sizeof(MYERROR));
   return rc;
 }
