@@ -42,7 +42,7 @@ do { \
 
 #define MYINFO_SET_USHORT(val) \
 do { \
-  *((SQLUSMALLINT *)num_info)= (val); \
+  *((SQLUSMALLINT *)num_info)= (SQLUSMALLINT)(val); \
   *value_len= sizeof(SQLUSMALLINT); \
   return SQL_SUCCESS; \
 } while(0)
@@ -133,7 +133,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
                      0);
 
   case SQL_COLLATION_SEQ:
-    MYINFO_SET_STR(dbc->mysql.charset->name);
+    MYINFO_SET_STR(dbc->mysql->charset->name);
 
   case SQL_COLUMN_ALIAS:
     MYINFO_SET_STR("Y");
@@ -199,7 +199,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_CREATE_VIEW:
     /** @todo SQL_CV_LOCAL ? */
-    if (is_minimum_version(dbc->mysql.server_version, "5.0"))
+    if (is_minimum_version(dbc->mysql->server_version, "5.0"))
       MYINFO_SET_ULONG(SQL_CV_CREATE_VIEW | SQL_CV_CHECK_OPTION |
                        SQL_CV_CASCADED);
     else
@@ -227,10 +227,10 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_DATABASE_NAME:
     if (is_connected(dbc) && reget_current_catalog(dbc))
-      return set_dbc_error(dbc, "HY000",
+      return dbc->set_error("HY000",
                            "SQLGetInfo() failed to return current catalog.",
                            0);
-    MYINFO_SET_STR(dbc->database ? dbc->database : "null");
+    MYINFO_SET_STR(!dbc->database.empty() ? dbc->database.c_str() : "null");
 
   case SQL_DATETIME_LITERALS:
     MYINFO_SET_ULONG(SQL_DL_SQL92_DATE | SQL_DL_SQL92_TIME |
@@ -241,7 +241,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 
   case SQL_DBMS_VER:
     /** @todo technically this is not right: should be ##.##.#### */
-    MYINFO_SET_STR(dbc->mysql.server_version);
+    MYINFO_SET_STR(dbc->mysql->server_version);
 
   case SQL_DDL_INDEX:
     MYINFO_SET_ULONG(SQL_DI_CREATE_INDEX | SQL_DI_DROP_INDEX);
@@ -284,7 +284,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     MYINFO_SET_ULONG(SQL_DT_DROP_TABLE | SQL_DT_CASCADE | SQL_DT_RESTRICT);
 
   case SQL_DROP_VIEW:
-    if (is_minimum_version(dbc->mysql.server_version, "5.0"))
+    if (is_minimum_version(dbc->mysql->server_version, "5.0"))
       MYINFO_SET_ULONG(SQL_DV_DROP_VIEW | SQL_DV_CASCADE | SQL_DV_RESTRICT);
     else
       MYINFO_SET_ULONG(0);
@@ -359,7 +359,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     We have INFORMATION_SCHEMA.SCHEMATA, but we don't report it
     because the driver exposes databases (schema) as catalogs.
     */
-    if (is_minimum_version(dbc->mysql.server_version, "5.1"))
+    if (is_minimum_version(dbc->mysql->server_version, "5.1"))
       MYINFO_SET_ULONG(SQL_ISV_CHARACTER_SETS | SQL_ISV_COLLATIONS |
                        SQL_ISV_COLUMN_PRIVILEGES | SQL_ISV_COLUMNS |
                        SQL_ISV_KEY_COLUMN_USAGE |
@@ -367,7 +367,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
                        /* SQL_ISV_SCHEMATA | */ SQL_ISV_TABLE_CONSTRAINTS |
                        SQL_ISV_TABLE_PRIVILEGES | SQL_ISV_TABLES |
                        SQL_ISV_VIEWS);
-    else if (is_minimum_version(dbc->mysql.server_version, "5.0"))
+    else if (is_minimum_version(dbc->mysql->server_version, "5.0"))
       MYINFO_SET_ULONG(SQL_ISV_CHARACTER_SETS | SQL_ISV_COLLATIONS |
                        SQL_ISV_COLUMN_PRIVILEGES | SQL_ISV_COLUMNS |
                        SQL_ISV_KEY_COLUMN_USAGE | /* SQL_ISV_SCHEMATA | */
@@ -393,7 +393,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     the MySQL Reference Manual (which is, in turn, generated from the source)
     with the pre-reserved ODBC keywords removed.
     */
-    if (is_minimum_version(dbc->mysql.server_version, "5.7"))
+    if (is_minimum_version(dbc->mysql->server_version, "8.0.22"))
       MYINFO_SET_STR("ACCESSIBLE,ANALYZE,ASENSITIVE,BEFORE,BIGINT,BINARY,BLOB,"
                      "CALL,CHANGE,CONDITION,DATABASE,DATABASES,DAY_HOUR,"
                      "DAY_MICROSECOND,DAY_MINUTE,DAY_SECOND,DELAYED,"
@@ -404,21 +404,21 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
                      "INT2,INT3,INT4,INT8,IO_AFTER_GTIDS,IO_BEFORE_GTIDS,"
                      "ITERATE,KEYS,KILL,LEAVE,LIMIT,LINEAR,LINES,LOAD,LOCALTIME,"
                      "LOCALTIMESTAMP,LOCK,LONG,LONGBLOB,LONGTEXT,LOOP,"
-                     "LOW_PRIORITY,MASTER_BIND,MASTER_HEARTBEAT_PERIOD,"
-                     "MASTER_SSL_VERIFY_SERVER_CERT,MAXVALUE,MEDIUMBLOB,"
+                     "LOW_PRIORITY,SOURCE_BIND,SOURCE_HEARTBEAT_PERIOD,"
+                     "SOURCE_SSL_VERIFY_SERVER_CERT,MAXVALUE,MEDIUMBLOB,"
                      "MEDIUMINT,MEDIUMTEXT,MIDDLEINT,MINUTE_MICROSECOND,"
                      "MINUTE_SECOND,MOD,MODIFIES,NO_WRITE_TO_BINLOG,NONBLOCKING,ONE_SHOT,"
                      "OPTIMIZE,OPTIONALLY,OUT,OUTFILE,PARTITION,PURGE,RANGE,"
                      "READ_ONLY,READS,READ_WRITE,REGEXP,RELEASE,RENAME,REPEAT,"
                      "REPLACE,REQUIRE,RESIGNAL,RETURN,RLIKE,SCHEMAS,"
                      "SECOND_MICROSECOND,SENSITIVE,SEPARATOR,SHOW,SIGNAL,SLOW,"
-                     "SPATIAL,SPECIFIC,SQL_AFTER_GTIDS,SQL_BEFORE_GTIDS"
+                     "SPATIAL,SPECIFIC,SQL_AFTER_GTIDS,SQL_BEFORE_GTIDS,"
                      "SQL_BIG_RESULT,SQL_CALC_FOUND_ROWS,SQLEXCEPTION,"
                      "SQL_SMALL_RESULT,SSL,STARTING,STRAIGHT_JOIN,TERMINATED,"
                      "TINYBLOB,TINYINT,TINYTEXT,TRIGGER,UNDO,UNLOCK,UNSIGNED,"
                      "USE,UTC_DATE,UTC_TIME,UTC_TIMESTAMP,VARBINARY,"
                      "VARCHARACTER,WHILE,X509,XOR,YEAR_MONTH,ZEROFILL");
-    else if (is_minimum_version(dbc->mysql.server_version, "5.6"))
+    else if (is_minimum_version(dbc->mysql->server_version, "5.7"))
       MYINFO_SET_STR("ACCESSIBLE,ANALYZE,ASENSITIVE,BEFORE,BIGINT,BINARY,BLOB,"
                      "CALL,CHANGE,CONDITION,DATABASE,DATABASES,DAY_HOUR,"
                      "DAY_MICROSECOND,DAY_MINUTE,DAY_SECOND,DELAYED,"
@@ -429,21 +429,46 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
                      "INT2,INT3,INT4,INT8,IO_AFTER_GTIDS,IO_BEFORE_GTIDS,"
                      "ITERATE,KEYS,KILL,LEAVE,LIMIT,LINEAR,LINES,LOAD,LOCALTIME,"
                      "LOCALTIMESTAMP,LOCK,LONG,LONGBLOB,LONGTEXT,LOOP,"
-                     "LOW_PRIORITY,MASTER_BIND,MASTER_HEARTBEAT_PERIOD,"
-                     "MASTER_SSL_VERIFY_SERVER_CERT,MAXVALUE,MEDIUMBLOB,"
+                     "LOW_PRIORITY,\x4D\x41\x53\x54\x45\x52_BIND,\x4D\x41\x53\x54\x45\x52_HEARTBEAT_PERIOD,"
+                     "\x4D\x41\x53\x54\x45\x52_SSL_VERIFY_SERVER_CERT,MAXVALUE,MEDIUMBLOB,"
+                     "MEDIUMINT,MEDIUMTEXT,MIDDLEINT,MINUTE_MICROSECOND,"
+                     "MINUTE_SECOND,MOD,MODIFIES,NO_WRITE_TO_BINLOG,NONBLOCKING,ONE_SHOT,"
+                     "OPTIMIZE,OPTIONALLY,OUT,OUTFILE,PARTITION,PURGE,RANGE,"
+                     "READ_ONLY,READS,READ_WRITE,REGEXP,RELEASE,RENAME,REPEAT,"
+                     "REPLACE,REQUIRE,RESIGNAL,RETURN,RLIKE,SCHEMAS,"
+                     "SECOND_MICROSECOND,SENSITIVE,SEPARATOR,SHOW,SIGNAL,SLOW,"
+                     "SPATIAL,SPECIFIC,SQL_AFTER_GTIDS,SQL_BEFORE_GTIDS,"
+                     "SQL_BIG_RESULT,SQL_CALC_FOUND_ROWS,SQLEXCEPTION,"
+                     "SQL_SMALL_RESULT,SSL,STARTING,STRAIGHT_JOIN,TERMINATED,"
+                     "TINYBLOB,TINYINT,TINYTEXT,TRIGGER,UNDO,UNLOCK,UNSIGNED,"
+                     "USE,UTC_DATE,UTC_TIME,UTC_TIMESTAMP,VARBINARY,"
+                     "VARCHARACTER,WHILE,X509,XOR,YEAR_MONTH,ZEROFILL");
+    else if (is_minimum_version(dbc->mysql->server_version, "5.6"))
+      MYINFO_SET_STR("ACCESSIBLE,ANALYZE,ASENSITIVE,BEFORE,BIGINT,BINARY,BLOB,"
+                     "CALL,CHANGE,CONDITION,DATABASE,DATABASES,DAY_HOUR,"
+                     "DAY_MICROSECOND,DAY_MINUTE,DAY_SECOND,DELAYED,"
+                     "DETERMINISTIC,DISTINCTROW,DIV,DUAL,EACH,ELSEIF,ENCLOSED,"
+                     "ESCAPED,EXIT,EXPLAIN,FLOAT4,FLOAT8,FORCE,FULLTEXT,GENERAL,"
+                     "GET,HIGH_PRIORITY,HOUR_MICROSECOND,HOUR_MINUTE,"
+                     "HOUR_SECOND,IF,IGNORE,IGNORE_SERVER_IDS,INFILE,INOUT,INT1,"
+                     "INT2,INT3,INT4,INT8,IO_AFTER_GTIDS,IO_BEFORE_GTIDS,"
+                     "ITERATE,KEYS,KILL,LEAVE,LIMIT,LINEAR,LINES,LOAD,LOCALTIME,"
+                     "LOCALTIMESTAMP,LOCK,LONG,LONGBLOB,LONGTEXT,LOOP,"
+                     "LOW_PRIORITY,\x4D\x41\x53\x54\x45\x52_BIND,\x4D\x41\x53\x54\x45\x52_HEARTBEAT_PERIOD,"
+                     "\x4D\x41\x53\x54\x45\x52_SSL_VERIFY_SERVER_CERT,MAXVALUE,MEDIUMBLOB,"
                      "MEDIUMINT,MEDIUMTEXT,MIDDLEINT,MINUTE_MICROSECOND,"
                      "MINUTE_SECOND,MOD,MODIFIES,NO_WRITE_TO_BINLOG,ONE_SHOT,"
                      "OPTIMIZE,OPTIONALLY,OUT,OUTFILE,PARTITION,PURGE,RANGE,"
                      "READ_ONLY,READS,READ_WRITE,REGEXP,RELEASE,RENAME,REPEAT,"
                      "REPLACE,REQUIRE,RESIGNAL,RETURN,RLIKE,SCHEMAS,"
                      "SECOND_MICROSECOND,SENSITIVE,SEPARATOR,SHOW,SIGNAL,SLOW,"
-                     "SPATIAL,SPECIFIC,SQL_AFTER_GTIDS,SQL_BEFORE_GTIDS"
+                     "SPATIAL,SPECIFIC,SQL_AFTER_GTIDS,SQL_BEFORE_GTIDS,"
                      "SQL_BIG_RESULT,SQL_CALC_FOUND_ROWS,SQLEXCEPTION,"
                      "SQL_SMALL_RESULT,SSL,STARTING,STRAIGHT_JOIN,TERMINATED,"
                      "TINYBLOB,TINYINT,TINYTEXT,TRIGGER,UNDO,UNLOCK,UNSIGNED,"
                      "USE,UTC_DATE,UTC_TIME,UTC_TIMESTAMP,VARBINARY,"
                      "VARCHARACTER,WHILE,X509,XOR,YEAR_MONTH,ZEROFILL");
-    else if (is_minimum_version(dbc->mysql.server_version, "5.5"))
+    else if (is_minimum_version(dbc->mysql->server_version, "5.5"))
       MYINFO_SET_STR("ACCESSIBLE,ANALYZE,ASENSITIVE,BEFORE,BIGINT,BINARY,BLOB,"
                      "CALL,CHANGE,CONDITION,DATABASE,DATABASES,DAY_HOUR,"
                      "DAY_MICROSECOND,DAY_MINUTE,DAY_SECOND,DELAYED,"
@@ -453,8 +478,8 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
                      "IF,IGNORE,IGNORE_SERVER_IDS,INFILE,INOUT,INT1,INT2,INT3,"
                      "INT4,INT8,ITERATE,KEYS,KILL,LEAVE,LIMIT,LINEAR,LINES,"
                      "LOAD,LOCALTIME,LOCALTIMESTAMP,LOCK,LONG,LONGBLOB,"
-                     "LONGTEXT,LOOP,LOW_PRIORITY,MASTER_HEARTBEAT_PERIOD,"
-                     "MASTER_SSL_VERIFY_SERVER_CERT,MAXVALUE,MEDIUMBLOB,"
+                     "LONGTEXT,LOOP,LOW_PRIORITY,\x4D\x41\x53\x54\x45\x52_HEARTBEAT_PERIOD,"
+                     "\x4D\x41\x53\x54\x45\x52_SSL_VERIFY_SERVER_CERT,MAXVALUE,MEDIUMBLOB,"
                      "MEDIUMINT,MEDIUMTEXT,MIDDLEINT,MINUTE_MICROSECOND,"
                      "MINUTE_SECOND,MOD,MODIFIES,NO_WRITE_TO_BINLOG,OPTIMIZE,"
                      "OPTIONALLY,OUT,OUTFILE,PURGE,RANGE,READ_ONLY,READS,"
@@ -466,7 +491,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
                      "TINYBLOB,TINYINT,TINYTEXT,TRIGGER,UNDO,UNLOCK,UNSIGNED,"
                      "USE,UTC_DATE,UTC_TIME,UTC_TIMESTAMP,VARBINARY,"
                      "VARCHARACTER,WHILE,X509,XOR,YEAR_MONTH,ZEROFILL");
-    else if (is_minimum_version(dbc->mysql.server_version, "5.1"))
+    else if (is_minimum_version(dbc->mysql->server_version, "5.1"))
       MYINFO_SET_STR("ACCESSIBLE,ANALYZE,ASENSITIVE,BEFORE,BIGINT,BINARY,BLOB,"
                      "CALL,CHANGE,CONDITION,DATABASE,DATABASES,DAY_HOUR,"
                      "DAY_MICROSECOND,DAY_MINUTE,DAY_SECOND,DELAYED,"
@@ -476,7 +501,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
                      "IF,IGNORE,INFILE,INOUT,INT1,INT2,INT3,INT4,INT8,ITERATE,"
                      "KEYS,KILL,LEAVE,LIMIT,LINEAR,LINES,LOAD,LOCALTIME,"
                      "LOCALTIMESTAMP,LOCK,LONG,LONGBLOB,LONGTEXT,LOOP,"
-                     "LOW_PRIORITY,MASTER_SSL_VERIFY_SERVER_CERT,MEDIUMBLOB,"
+                     "LOW_PRIORITY,\x4D\x41\x53\x54\x45\x52_SSL_VERIFY_SERVER_CERT,MEDIUMBLOB,"
                      "MEDIUMINT,MEDIUMTEXT,MIDDLEINT,MINUTE_MICROSECOND,"
                      "MINUTE_SECOND,MOD,MODIFIES,NO_WRITE_TO_BINLOG,OPTIMIZE,"
                      "OPTIONALLY,OUT,OUTFILE,PURGE,RANGE,READ_ONLY,READS,"
@@ -488,7 +513,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
                      "TINYTEXT,TRIGGER,UNDO,UNLOCK,UNSIGNED,USE,UTC_DATE,"
                      "UTC_TIME,UTC_TIMESTAMP,VARBINARY,VARCHARACTER,WHILE,X509,"
                      "XOR,YEAR_MONTH,ZEROFILL");
-    else if (is_minimum_version(dbc->mysql.server_version, "5.0"))
+    else if (is_minimum_version(dbc->mysql->server_version, "5.0"))
       MYINFO_SET_STR("ANALYZE,ASENSITIVE,BEFORE,BIGINT,BINARY,BLOB,CALL,CHANGE,"
                      "CONDITION,DATABASE,DATABASES,DAY_HOUR,DAY_MICROSECOND,"
                      "DAY_MINUTE,DAY_SECOND,DELAYED,DETERMINISTIC,DISTINCTROW,"
@@ -536,8 +561,12 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   case SQL_MAX_BINARY_LITERAL_LEN:
     MYINFO_SET_ULONG(0);
 
+  // SQL_MAX_QUALIFIER_NAME_LEN is also defined as SQL_MAX_CATALOG_NAME_LEN
   case SQL_MAX_CATALOG_NAME_LEN:
-    MYINFO_SET_USHORT(NAME_LEN);
+    if (!dbc->ds->no_catalog)
+      MYINFO_SET_USHORT(64);
+    else
+      MYINFO_SET_USHORT(0);
 
   case SQL_MAX_CHAR_LITERAL_LEN:
     MYINFO_SET_ULONG(0);
@@ -573,7 +602,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     MYINFO_SET_USHORT(NAME_LEN);
 
   case SQL_MAX_INDEX_SIZE:
-    if (is_minimum_version(dbc->mysql.server_version, "5.0"))
+    if (is_minimum_version(dbc->mysql->server_version, "5.0"))
       MYINFO_SET_USHORT(3072);
     else
       MYINFO_SET_USHORT(1024);
@@ -587,8 +616,9 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   case SQL_MAX_ROW_SIZE_INCLUDES_LONG:
     MYINFO_SET_STR("Y");
 
+  // SQL_MAX_OWNER_NAME_LEN is also defined as SQL_MAX_SCHEMA_NAME_LEN
   case SQL_MAX_SCHEMA_NAME_LEN:
-    if (!dbc->ds->no_catalog)
+    if (!dbc->ds->no_schema)
       MYINFO_SET_USHORT(64);
     else
       MYINFO_SET_USHORT(0);
@@ -600,7 +630,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     MYINFO_SET_USHORT(NAME_LEN);
 
   case SQL_MAX_TABLES_IN_SELECT:
-    if (is_minimum_version(dbc->mysql.server_version, "5.0"))
+    if (is_minimum_version(dbc->mysql->server_version, "5.0"))
       MYINFO_SET_USHORT(63);
     else
       MYINFO_SET_USHORT(31);
@@ -658,13 +688,13 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     MYINFO_SET_ULONG(SQL_PAS_NO_BATCH);
 
   case SQL_PROCEDURE_TERM:
-    if (is_minimum_version(dbc->mysql.server_version, "5.0"))
+    if (is_minimum_version(dbc->mysql->server_version, "5.0"))
       MYINFO_SET_STR("stored procedure");
     else
       MYINFO_SET_STR("");
 
   case SQL_PROCEDURES:
-    if (is_minimum_version(dbc->mysql.server_version, "5.0"))
+    if (is_minimum_version(dbc->mysql->server_version, "5.0"))
       MYINFO_SET_STR("Y");
     else
       MYINFO_SET_STR("N");
@@ -683,11 +713,10 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     MYINFO_SET_STR("N");
 
   case SQL_SCHEMA_TERM:
-    /* This is because we map MySQL database (schema) to catalog. */
-    MYINFO_SET_STR("");
+    MYINFO_SET_STR((dbc->ds && dbc->ds->no_schema) ? "" : "database");
 
   case SQL_SCHEMA_USAGE:
-    if (!dbc->ds->no_catalog)
+    if (!dbc->ds->no_schema)
       MYINFO_SET_ULONG(SQL_SU_DML_STATEMENTS | SQL_SU_PROCEDURE_INVOCATION |
                        SQL_SU_TABLE_DEFINITION | SQL_SU_INDEX_DEFINITION |
                        SQL_SU_PRIVILEGE_DEFINITION);
@@ -704,7 +733,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     MYINFO_SET_STR("\\");
 
   case SQL_SERVER_NAME:
-    MYINFO_SET_STR(dbc->mysql.host_info);
+    MYINFO_SET_STR(dbc->mysql->host_info);
 
   case SQL_SPECIAL_CHARACTERS:
     /* We can handle anything but / and \xff. */
@@ -1068,7 +1097,7 @@ SQLRETURN SQL_API MySQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType)
   {
     stmt_result_free(stmt);
     x_free(stmt->result_array);
-    return set_stmt_error(stmt, "S1001", "Not enough memory", 4001);
+    return stmt->set_error("S1001", "Not enough memory", 4001);
   }
 
   if (fSqlType == SQL_ALL_TYPES)
