@@ -38,8 +38,8 @@
 /* {{{ my_l_to_a() -I- */
 static char * my_l_to_a(char * buf, size_t buf_size, long long a)
 {
-	myodbc_snprintf(buf, buf_size, "%lld", (long long) a);
-	return buf;
+  myodbc_snprintf(buf, buf_size, "%lld", (long long) a);
+  return buf;
 }
 /* }}} */
 
@@ -47,8 +47,8 @@ static char * my_l_to_a(char * buf, size_t buf_size, long long a)
 /* {{{ my_ul_to_a() -I- */
 static char * my_ul_to_a(char * buf, size_t buf_size, unsigned long long a)
 {
-	myodbc_snprintf(buf, buf_size, "%llu", (unsigned long long) a);
-	return buf;
+  myodbc_snprintf(buf, buf_size, "%llu", (unsigned long long) a);
+  return buf;
 }
 /* }}} */
 
@@ -56,8 +56,8 @@ static char * my_ul_to_a(char * buf, size_t buf_size, unsigned long long a)
 /* {{{ my_f_to_a() -I- */
 static char * my_f_to_a(char * buf, size_t buf_size, double a)
 {
-	myodbc_snprintf(buf, buf_size, "%f", a);
-	return buf;
+  myodbc_snprintf(buf, buf_size, "%.17e", a);
+  return buf;
 }
 /* }}} */
 
@@ -65,9 +65,9 @@ static char * my_f_to_a(char * buf, size_t buf_size, double a)
 /* {{{ ssps_init() -I- */
 void ssps_init(STMT *stmt)
 {
-  stmt->ssps= mysql_stmt_init(&stmt->dbc->mysql);
+  stmt->ssps= mysql_stmt_init(stmt->dbc->mysql);
 
-  stmt->result_bind= 0;
+  stmt->result_bind = 0;
 }
 /* }}} */
 
@@ -98,7 +98,7 @@ BOOL ssps_get_out_params(STMT *stmt)
   if (is_call_procedure(&stmt->query))
   {
     MYSQL_ROW values= NULL;
-    DESCREC   *iprec, *aprec;
+    DESCREC   *iprec, *aprec, *irrec;
     uint      counter= 0;
     int       i, out_params;
 
@@ -109,7 +109,14 @@ BOOL ssps_get_out_params(STMT *stmt)
     /* Thus function interface has to be changed */
     if (stmt->ssps_bind_result() == 0)
     {
-      values= fetch_row(stmt);
+      try
+      {
+        values = stmt->fetch_row();
+      }
+      catch(MYERROR &e)
+      {
+        return FALSE;
+      }
 
       /* We need this for fetch_varlength_columns pointed by fix_fields, so it omits
          streamed parameters */
@@ -141,7 +148,7 @@ BOOL ssps_get_out_params(STMT *stmt)
       if (out_params)
       {
         for (i= 0;
-             i < myodbc_min(stmt->ipd->count, stmt->apd->count) && counter < field_count(stmt);
+             i < myodbc_min(stmt->ipd->rcount(), stmt->apd->rcount()) && counter < stmt->field_count();
              ++i)
         {
           /* Making bit field look "normally" */
@@ -251,13 +258,50 @@ BOOL ssps_get_out_params(STMT *stmt)
 
 int ssps_get_result(STMT *stmt)
 {
-  if (stmt->result)
+  try
   {
-    if (!if_forward_cache(stmt))
+    if (stmt->result)
     {
-      return mysql_stmt_store_result(stmt->ssps);
-    }
+      if (!if_forward_cache(stmt))
+      {
+        return mysql_stmt_store_result(stmt->ssps);
+      }
+      else
+      {
+        /*
+          There is no way of telling beforehand if the result set is the
+          normal result set or out parameters.
 
+          In order to get the server status GOT_OUT_PARAMETERS we need
+          to read at least two rows from the result set.
+          1st row is the data for the OUT parameters, 2nd read attempt should
+          return no data and pick up the EOF and the server status.
+        */
+
+        size_t field_count = stmt->field_count();
+        // Try fetching 1st row, return if no data is available.
+        if(stmt->fetch_row(true) == nullptr)
+          return 0;
+        stmt->m_row_storage.set_size(1, field_count);
+        stmt->m_row_storage.set_data(stmt->result_bind);
+        // Add 2nd row if it is fetched
+        if (stmt->fetch_row(true))
+        {
+          stmt->m_row_storage.next_row();
+          stmt->m_row_storage.set_data(stmt->result_bind);
+        }
+        // Set row counter to start before reading rows
+        stmt->m_row_storage.first_row();
+      }
+    }
+  }
+  catch(MYERROR &e)
+  {
+    return e.retcode;
+  }
+  catch(...)
+  {
+    return SQL_ERROR;
   }
 
   return 0;
@@ -268,7 +312,7 @@ void free_result_bind(STMT *stmt)
 {
   if (stmt->result_bind != NULL)
   {
-    int i, field_cnt= field_count(stmt);
+    int i, field_cnt = stmt->field_count();
 
     x_free(stmt->result_bind[0].is_null);
     x_free(stmt->result_bind[0].length);
@@ -329,11 +373,11 @@ SQLRETURN ssps_fetch_chunk(STMT *stmt, char *dest, unsigned long dest_bytes, uns
     {
       case  CR_INVALID_PARAMETER_NO:
         /* Shouldn't really happen here*/
-        return set_stmt_error(stmt, "07009", "Invalid descriptor index", 0);
+        return stmt->set_error("07009", "Invalid descriptor index", 0);
 
       case CR_NO_DATA: return SQL_NO_DATA;
 
-      default: set_stmt_error(stmt, "HY000", "Internal error", 0);
+      default: stmt->set_error("HY000", "Internal error", 0);
     }
   }
   else
@@ -343,7 +387,7 @@ SQLRETURN ssps_fetch_chunk(STMT *stmt, char *dest, unsigned long dest_bytes, uns
 
     if (*bind.error)
     {
-      set_stmt_error(stmt, "01004", NULL, 0);
+      stmt->set_error("01004", NULL, 0);
       return SQL_SUCCESS_WITH_INFO;
     }
 
@@ -367,7 +411,8 @@ bool is_varlen_type(enum enum_field_types type)
           type == MYSQL_TYPE_TINY_BLOB ||
           type == MYSQL_TYPE_MEDIUM_BLOB ||
           type == MYSQL_TYPE_LONG_BLOB ||
-          type == MYSQL_TYPE_VAR_STRING);
+          type == MYSQL_TYPE_VAR_STRING ||
+          type == MYSQL_TYPE_JSON);
 }
 
 /* The structure and following allocation function are borrowed from c/c++ and adopted */
@@ -414,11 +459,19 @@ allocate_buffer_for_field(const MYSQL_FIELD * const field, BOOL outparams)
 
     case MYSQL_TYPE_INT24:
     case MYSQL_TYPE_LONG:
-    case MYSQL_TYPE_FLOAT:
       result.size=4;
       break;
 
+    /*
+      For consistency with results obtained through MYSQL_ROW
+      we must fetch FLOAT and DOUBLE as character data
+    */
     case MYSQL_TYPE_DOUBLE:
+    case MYSQL_TYPE_FLOAT:
+      result.size=24;
+      result.type = MYSQL_TYPE_STRING;
+      break;
+
     case MYSQL_TYPE_LONGLONG:
       result.size=8;
       break;
@@ -440,6 +493,7 @@ allocate_buffer_for_field(const MYSQL_FIELD * const field, BOOL outparams)
     case MYSQL_TYPE_BLOB:
     case MYSQL_TYPE_STRING:
     case MYSQL_TYPE_VAR_STRING:
+    case MYSQL_TYPE_JSON:
       /* We will get length with fetch and then fetch column */
       if (field->length > 0 && field->length < 1025)
         result.size= field->length + 1;
@@ -501,7 +555,7 @@ allocate_buffer_for_field(const MYSQL_FIELD * const field, BOOL outparams)
 
 static MYSQL_ROW fetch_varlength_columns(STMT *stmt, MYSQL_ROW values)
 {
-  const unsigned int  num_fields= field_count(stmt);
+  const unsigned int  num_fields = stmt->field_count();
   unsigned int i;
   uint desc_index= ~0L, stream_column= ~0L;
 
@@ -595,9 +649,113 @@ void STMT::alloc_lengths(size_t num)
   lengths_allocated = true;
 }
 
+void STMT::reset_setpos_apd()
+{
+  setpos_apd.reset();
+}
+
+bool STMT::is_dynamic_cursor()
+{
+  return stmt_options.cursor_type == SQL_CURSOR_DYNAMIC;
+}
+
+
+void STMT::free_unbind()
+{
+  ard->reset();
+}
+
+
+/*
+  Do only a "light" reset
+*/
+void STMT::reset()
+{
+  buf_set_pos(0);
+
+  // If data existed before invalidating the result array does not need freeing
+  if (m_row_storage.invalidate())
+    result_array = nullptr;
+}
+
+void STMT::free_reset_out_params()
+{
+  if (out_params_state == OPS_STREAMS_PENDING)
+  {
+    /* Magical out params fetch */
+    mysql_stmt_fetch(ssps);
+  }
+  out_params_state = OPS_UNKNOWN;
+  apd->free_paramdata();
+  /* reset data-at-exec state */
+  dae_type = 0;
+  scroller_reset(this);
+}
+
+void STMT::free_reset_params()
+{
+  if (param_bind != NULL)
+  {
+    reset_dynamic(param_bind);
+  }
+  if (ssps)
+  {
+    mysql_stmt_reset(ssps);
+  }
+  /* remove all params and reset count to 0 (per spec) */
+  /* http://msdn2.microsoft.com/en-us/library/ms709284.aspx */
+  // NOTE: check if this breaks anything
+  apd->records2.clear();
+}
+
+void STMT::free_fake_result(bool clear_all_results)
+{
+  if (!fake_result)
+  {
+    if (clear_all_results)
+    {
+      /* We seiously CLOSEing statement for preparing handle object for
+         new query */
+      alloc_root.Clear();
+      while (!next_result(this))
+      {
+        get_result_metadata(this, TRUE);
+      }
+    }
+  }
+  else
+  {
+    if (result && result->field_alloc
+#if (!MYSQL8)
+      && result->field_alloc->pre_alloc
+#endif
+      )
+    {
+      result->field_alloc->Clear();
+    }
+
+    stmt_result_free(this);
+  }
+
+}
+
 STMT::~STMT()
 {
   free_lengths();
+
+  if (ssps != NULL)
+  {
+    mysql_stmt_close(ssps);
+    ssps = NULL;
+  }
+
+  reset_setpos_apd();
+  delete_parsed_query(&query);
+  delete_parsed_query(&orig_query);
+  delete_param_bind(param_bind);
+
+  LOCK_DBC(dbc);
+  dbc->stmt_list.remove(this);
 }
 
 void STMT::reset_getdata_position()
@@ -617,6 +775,24 @@ SQLRETURN STMT::set_error(myodbc_errid errid, const char *errtext,
   return error.retcode;
 }
 
+SQLRETURN STMT::set_error(myodbc_errid errid)
+{
+  return set_error(errid, mysql_error(dbc->mysql), mysql_errno(dbc->mysql));
+}
+
+SQLRETURN STMT::set_error(const char *state, const char *msg,
+                          SQLINTEGER errcode)
+{
+    error = MYERROR(state, msg, errcode, dbc->st_error_prefix);
+    return error.retcode;
+}
+
+SQLRETURN STMT::set_error(const char *state)
+{
+  return set_error(state, mysql_error(dbc->mysql), mysql_errno(dbc->mysql));
+}
+
+
 long STMT::compute_cur_row(unsigned fFetchType, SQLLEN irow)
 {
   long cur_row = 0;
@@ -698,7 +874,7 @@ long STMT::compute_cur_row(unsigned fFetchType, SQLLEN irow)
       case SQL_NO_DATA:
         throw MYERROR(SQL_NO_DATA_FOUND);
       case SQL_ERROR:
-        set_error(MYERR_S1000, mysql_error(&dbc->mysql), 0);
+        set_error(MYERR_S1000, mysql_error(dbc->mysql), 0);
         throw error;
       }
     }
@@ -728,7 +904,7 @@ long STMT::compute_cur_row(unsigned fFetchType, SQLLEN irow)
 
 int STMT::ssps_bind_result()
 {
-  const unsigned int  num_fields= field_count(this);
+  const unsigned int num_fields = field_count();
   unsigned int        i;
 
   if (num_fields == 0)
@@ -804,34 +980,112 @@ int STMT::ssps_bind_result()
       }
     }
 
-    return mysql_stmt_bind_result(ssps, result_bind);
+    int rc = mysql_stmt_bind_result(ssps, result_bind);
+    if (rc)
+    {
+      const char *err = mysql_stmt_error(ssps);
+      set_error("HY000", err, 0);
+    }
+    return rc;
   }
 
   return 0;
 }
 
+SQLRETURN STMT::bind_query_attrs(bool use_ssps)
+{
+  if (use_ssps)
+  {
+    set_error(MYERR_01000,
+              "Query attributes for prepared statements are not supported",
+              0);
+    return SQL_SUCCESS_WITH_INFO;
+  }
+
+  uint rcount = (uint)apd->rcount();
+  if (rcount == param_count)
+  {
+    // Nothing to do
+    return SQL_SUCCESS;
+  }
+  else if (rcount < param_count)
+  {
+    set_error( MYERR_07001,
+              "The number of parameter markers is larger "
+              "than he number of parameters provided",0);
+    return SQL_ERROR;
+  }
+  else if (!dbc->has_query_attrs)
+  {
+    set_error(MYERR_01000,
+              "The server does not support query attributes",
+              0);
+    return SQL_SUCCESS_WITH_INFO;
+  }
+
+  uint num = param_count;
+  query_attr_bind.clear();
+  query_attr_bind.reserve(rcount - param_count);
+  query_attr_names.clear();
+  query_attr_names.reserve(rcount - param_count);
+
+  while(num < rcount)
+  {
+    DESCREC *aprec = desc_get_rec(apd, num, false);
+    DESCREC *iprec = desc_get_rec(ipd, num, false);
+
+    /*
+      IPREC and APREC should both be not NULL if query attributes
+      or parameters are set.
+    */
+    if (!aprec || !iprec)
+      return SQL_SUCCESS; // Nothing to do
+
+    query_attr_bind.emplace_back(MYSQL_BIND{});
+    MYSQL_BIND *bind = &query_attr_bind.back();
+
+    query_attr_names.emplace_back(iprec->par.val());
+
+    // This will just fill the bind structure and do the param data conversion
+    if(insert_param(this, bind, apd, aprec, iprec, 0) == SQL_ERROR)
+    {
+      set_error(MYERR_01000,
+                "The number of attributes is larger than the "
+                "number of attribute values provided",
+                0);
+      return SQL_ERROR;
+    }
+    ++num;
+  }
+
+  MYSQL_BIND *bind = query_attr_bind.data();
+  const char** names = (const char**)query_attr_names.data();
+
+  if (mysql_bind_param(dbc->mysql, rcount - param_count,
+                        query_attr_bind.data(),
+                        (const char**)query_attr_names.data()))
+  {
+    set_error("HY000");
+    return SQL_SUCCESS_WITH_INFO;
+  }
+
+  return SQL_SUCCESS;
+}
+
 /*
   Function determines if the buffers need to be extended
 */
 BOOL ssps_buffers_need_extending(STMT *stmt)
 {
-  //if (stmt->fix_fields == NULL)
-  //{
-  //  /* That is enough to tell that not */
-  //  return FALSE;
-  //}
-  //else
-  {
-    const unsigned int  num_fields= field_count(stmt);
-    unsigned int i;
+  const unsigned int  num_fields = stmt->field_count();
+  unsigned int i;
 
-    for (i= 0; i < num_fields; ++i)
+  for (i= 0; i < num_fields; ++i)
+  {
+    if (*stmt->result_bind[i].error != 0
+      && stmt->result_bind[i].buffer_length < (*stmt->result_bind[i].length))
     {
-      if (*stmt->result_bind[i].error != 0
-        && stmt->result_bind[i].buffer_length < (*stmt->result_bind[i].length))
-      {
-        return TRUE;
-      }
+      return TRUE;
     }
   }
 
@@ -946,6 +1200,7 @@ char * ssps_get_string(STMT *stmt, ulong column_number, char *value, ulong *leng
     case MYSQL_TYPE_BLOB:
     case MYSQL_TYPE_VARCHAR:
     case MYSQL_TYPE_VAR_STRING:
+    case MYSQL_TYPE_JSON:
       *length= *col_rbind->length;
       return (char *)(col_rbind->buffer);
     default:
@@ -1164,14 +1419,14 @@ SQLRETURN ssps_send_long_data(STMT *stmt, unsigned int param_number, const char
       /* We can fall back to assembling parameter's value on client */
         return SQL_SUCCESS_WITH_INFO;
       case CR_SERVER_GONE_ERROR:
-        return set_stmt_error(stmt, "08S01", mysql_stmt_error(stmt->ssps), 0);
+        return stmt->set_error("08S01", mysql_stmt_error(stmt->ssps), err);
       case CR_COMMANDS_OUT_OF_SYNC:
       case CR_UNKNOWN_ERROR:
-        return set_stmt_error(stmt, "HY000", mysql_stmt_error( stmt->ssps), 0);
+        return stmt->set_error("HY000", mysql_stmt_error( stmt->ssps), err);
       case CR_OUT_OF_MEMORY:
-        return set_stmt_error(stmt, "HY001", mysql_stmt_error(stmt->ssps), 0);
+        return stmt->set_error("HY001", mysql_stmt_error(stmt->ssps), err);
       default:
-        return set_stmt_error(stmt, "HY000", "unhandled error from mysql_stmt_send_long_data", 0 );
+        return stmt->set_error("HY000", "unhandled error from mysql_stmt_send_long_data", 0 );
     }
   }
 
