@@ -61,7 +61,7 @@ BOOL returned_result(STMT *stmt)
   }
   else
   {
-    return mysql_field_count(&stmt->dbc->mysql) > 0 ;
+    return mysql_field_count(stmt->dbc->mysql) > 0 ;
   }
 }
 
@@ -94,11 +94,11 @@ MYSQL_RES * stmt_get_result(STMT *stmt, BOOL force_use)
   /* We can't use USE_RESULT because SQLRowCount will fail in this case! */
   if (if_forward_cache(stmt) || force_use)
   {
-    return mysql_use_result(&stmt->dbc->mysql);
+    return mysql_use_result(stmt->dbc->mysql);
   }
   else
   {
-    return mysql_store_result(&stmt->dbc->mysql);
+    return mysql_store_result(stmt->dbc->mysql);
   }
 }
 
@@ -146,17 +146,17 @@ int get_result(STMT *stmt)
 }
 
 
-unsigned int field_count(STMT *stmt)
+size_t STMT::field_count()
 {
-  if (ssps_used(stmt))
+  if (ssps)
   {
-    return mysql_stmt_field_count(stmt->ssps);
+    return mysql_stmt_field_count(ssps);
   }
   else
   {
-    return stmt->result && stmt->result->field_count > 0 ?
-      stmt->result->field_count :
-      mysql_field_count(&stmt->dbc->mysql);
+    return result && result->field_count > 0 ?
+      result->field_count :
+      mysql_field_count(dbc->mysql);
   }
 }
 
@@ -170,7 +170,7 @@ my_ulonglong affected_rows(STMT *stmt)
   else
   {
     /* In some cases in c/odbc it cannot be used instead of mysql_num_rows */
-    return mysql_affected_rows(&stmt->dbc->mysql);
+    return mysql_affected_rows(stmt->dbc->mysql);
   }
 }
 
@@ -202,34 +202,45 @@ my_ulonglong num_rows(STMT *stmt)
 }
 
 
-MYSQL_ROW fetch_row(STMT *stmt)
+MYSQL_ROW STMT::fetch_row(bool read_unbuffered)
 {
-  if (ssps_used(stmt))
+  if (ssps)
   {
-    int error;
-    if (stmt->ssps_bind_result())
+    if (ssps_bind_result())
     {
-      return NULL;
+      return nullptr;
+    }
+    int err = 0;
+
+    if (read_unbuffered || m_row_storage.eof())
+    {
+      /* Reading results from network */
+      err = mysql_stmt_fetch(ssps);
+    }
+    else
+    {
+      /* Row is already buffered in row storage, use the row storage */
+      m_row_storage.fill_data(result_bind);
     }
 
-    error= mysql_stmt_fetch(stmt->ssps);
-
+    switch (err)
     {
-
-      if (error == MYSQL_NO_DATA)
+      case 1:
+        set_error("HY000", mysql_stmt_error(ssps),
+          mysql_stmt_errno(ssps));
+        throw error;
+      case MYSQL_NO_DATA:
         return nullptr;
-
-      if (stmt->fix_fields)
-      {
-        return stmt->fix_fields(stmt, nullptr); // it returns stmt->array
-      }
     }
 
-    return stmt->array;
+    if (fix_fields)
+      return fix_fields(this, nullptr); // it returns stmt->array
+
+    return array;
   }
   else
   {
-    return mysql_fetch_row(stmt->result);
+    return mysql_fetch_row(result);
   }
 }
 
@@ -296,7 +307,7 @@ int next_result(STMT *stmt)
   }
   else
   {
-    return mysql_next_result(&stmt->dbc->mysql);
+    return mysql_next_result(stmt->dbc->mysql);
   }
 }
 
@@ -372,7 +383,7 @@ BOOL is_null(STMT *stmt, ulong column_number, char *value)
    on a server. Returns SQLRETURN result code since preparing on client or
    server can produce errors, memory allocation to name one.  */
 SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length,
-          bool reset_sql_limit)
+          bool reset_sql_limit, bool force_prepare)
 {
   /* TODO: I guess we always have to have query length here */
   if (query_length <= 0)
@@ -394,8 +405,9 @@ SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length,
   stmt->param_count= PARAM_COUNT(&stmt->query);
   /* Trusting our parsing we are not using prepared statments unsless there are
      actually parameter markers in it */
-  if (!stmt->dbc->ds->no_ssps && PARAM_COUNT(&stmt->query) && !IS_BATCH(&stmt->query)
-    && preparable_on_server(&stmt->query, stmt->dbc->mysql.server_version))
+  if (!stmt->dbc->ds->no_ssps && (PARAM_COUNT(&stmt->query) || force_prepare)
+    && !IS_BATCH(&stmt->query)
+    && preparable_on_server(&stmt->query, stmt->dbc->mysql->server_version))
   {
     MYLOG_QUERY(stmt, "Using prepared statement");
     ssps_init(stmt);
@@ -404,22 +416,20 @@ SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length,
        it at the moment */
     if (!get_cursor_name(&stmt->query))
     {
-     myodbc_mutex_lock(&stmt->dbc->lock);
+     LOCK_STMT(stmt);
 
      if (reset_sql_limit)
         set_sql_select_limit(stmt->dbc, 0, false);
 
      int prep_res = mysql_stmt_prepare(stmt->ssps, query, query_length);
-     myodbc_mutex_unlock(&stmt->dbc->lock);
 
      if (prep_res)
       {
-        MYLOG_QUERY(stmt, mysql_error(&stmt->dbc->mysql));
+        MYLOG_QUERY(stmt, mysql_error(stmt->dbc->mysql));
 
-        set_stmt_error(stmt,"HY000",mysql_error(&stmt->dbc->mysql),
-                       mysql_errno(&stmt->dbc->mysql));
+        stmt->set_error("HY000");
         translate_error(stmt->error.sqlstate,MYERR_S1000,
-                        mysql_errno(&stmt->dbc->mysql));
+                        mysql_errno(stmt->dbc->mysql));
 
         return SQL_ERROR;
       }
@@ -463,42 +473,6 @@ SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length,
   return SQL_SUCCESS;
 }
 
-
-SQLRETURN append2param_value(STMT *stmt, DESCREC * aprec, const char *chunk, unsigned long length)
-{
-  if ( aprec->par.value )
-  {
-    /* Append to old value */
-    assert(aprec->par.alloced);
-    if ( !(aprec->par.value= (char*)myodbc_realloc(aprec->par.value,
-                                        aprec->par.value_length + length + 1,
-                                        MYF(0))) )
-    {
-      return stmt->set_error(MYERR_S1001,NULL,4001);
-    }
-
-    memcpy(aprec->par.value+aprec->par.value_length,chunk,length);
-    aprec->par.value_length+= length;
-    aprec->par.value[aprec->par.value_length]= 0;
-    aprec->par.alloced= TRUE;
-  }
-  else
-  {
-    /* New value */
-    if ( !(aprec->par.value= (char*)myodbc_malloc(length+1,MYF(0))) )
-    {
-      return stmt->set_error(MYERR_S1001,NULL,4001);
-    }
-    memcpy(aprec->par.value,chunk,length);
-    aprec->par.value_length= length;
-    aprec->par.value[aprec->par.value_length]= 0;
-    aprec->par.alloced= TRUE;
-  }
-
-  return SQL_SUCCESS;
-}
-
-
 SQLRETURN send_long_data (STMT *stmt, unsigned int param_num, DESCREC * aprec, const char *chunk,
                           unsigned long length)
 {
@@ -526,7 +500,8 @@ SQLRETURN send_long_data (STMT *stmt, unsigned int param_num, DESCREC * aprec, c
   else
 #endif
   {
-    return append2param_value(stmt, aprec, chunk, length);
+    aprec->par.add_param_data(chunk, length);
+    return SQL_SUCCESS;
   }
 }
 
@@ -607,7 +582,6 @@ void scroller_create(STMT * stmt, char *query, SQLULEN query_len)
 
   stmt->scroller.next_offset= myodbc_max(limit.offset, 0);
 
-  /*extend_buffer(&stmt->dbc->mysql.net, stmt->query_end, len2add);*/
   stmt->scroller.query_len= query_len + len2add;
   stmt->scroller.query= (char*)myodbc_malloc((size_t)stmt->scroller.query_len + 1,
                                           MYF(MY_ZEROFILL));
@@ -671,20 +645,14 @@ SQLRETURN scroller_prefetch(STMT * stmt)
 
   MYLOG_QUERY(stmt, stmt->scroller.query);
 
-  myodbc_mutex_lock(&stmt->dbc->lock);
+  LOCK_STMT(stmt);
 
   if (exec_stmt_query(stmt, stmt->scroller.query,
                         (unsigned long)stmt->scroller.query_len, FALSE))
   {
-    myodbc_mutex_unlock(&stmt->dbc->lock);
     return SQL_ERROR;
   }
-
   get_result_metadata(stmt, FALSE);
-
-  /* I think there is no need to do fix_result_types here */
-  myodbc_mutex_unlock(&stmt->dbc->lock);
-
   return SQL_SUCCESS;
 }
 
