@@ -1,30 +1,30 @@
-// Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of <MySQL Product>, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /**
   @file  parse.c
@@ -169,7 +169,7 @@ MY_PARSED_QUERY * reset_parsed_query(MY_PARSED_QUERY *pq, char * query,
     pq->query= query;
 
     if (pq->query == NULL)
-    { 
+    {
       pq->cs=         NULL;
       pq->query_end=  NULL;
     }
@@ -520,17 +520,17 @@ BOOL is_call_procedure(const MY_PARSED_QUERY * query)
             is likely to result in reading only (SELECT || SHOW).
 
             Some ODBC calls require knowledge about a statement
-            which we can not determine until we have executed 
+            which we can not determine until we have executed
             the statement. This is because we do not parse the SQL
             - the server does.
 
             However if we silently execute a pending statement we
             may insert rows.
 
-            So we do a very crude check of the SQL here to reduce 
+            So we do a very crude check of the SQL here to reduce
             the chance of a problem.
 
-    \sa     BUG 5778            
+    \sa     BUG 5778
 */
 BOOL stmt_returns_result(const MY_PARSED_QUERY *query)
 {
@@ -545,7 +545,7 @@ BOOL stmt_returns_result(const MY_PARSED_QUERY *query)
 /* TRUE if end has been reached */
 BOOL skip_spaces(MY_PARSER *parser)
 {
-  while(END_NOT_REACHED(parser) && IS_SPACE(parser))
+  while(END_NOT_REACHED(parser) && (IS_SPACE(parser) || IS_SPL_CHAR(parser)))
   {
     step_char(parser);
   }
@@ -556,12 +556,12 @@ BOOL skip_spaces(MY_PARSER *parser)
 
 BOOL skip_comment(MY_PARSER *parser)
 {
-  while(END_NOT_REACHED(parser) && 
-        ((parser->hash_comment && 
-            !compare(parser, &parser->syntax->new_line_end)) ||  
-          (parser->dash_comment && 
+  while(END_NOT_REACHED(parser) &&
+        ((parser->hash_comment &&
             !compare(parser, &parser->syntax->new_line_end)) ||
-          (parser->c_style_comment &&  
+          (parser->dash_comment &&
+            !compare(parser, &parser->syntax->new_line_end)) ||
+          (parser->c_style_comment &&
             !compare(parser, &parser->syntax->c_style_close_comment))))
   {
     step_char(parser);
@@ -744,7 +744,7 @@ BOOL is_query_separator(MY_PARSER *parser)
 BOOL tokenize(MY_PARSER *parser)
 {
   /* TODO: token info should contain length of a token */
-  
+
   skip_spaces(parser);
   /* 1st token - otherwise we lose it if it is on 0 position without spaces
      ahead of it */
@@ -906,7 +906,7 @@ BOOL case_compare(MY_PARSED_QUERY *pq, const char *pos, const MY_STRING *str)
 BOOL parse(MY_PARSED_QUERY *pq)
 {
   MY_PARSER parser;
-    
+
   init_parser(&parser, pq);
 
   if (tokenize(&parser))
@@ -968,7 +968,7 @@ BOOL remove_braces(MY_PARSER *parser)
                                   TOKEN_COUNT(parser->query) - 1);
         }
       }
-      
+
       /* Not the last char form now on */
       parser->query->last_char= NULL;
 
