@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,12 +36,12 @@
 
 #include <string.h>
 
-#if !defined(DBUG_OFF)
+#if !defined(NDEBUG)
 #include <assert.h>  // IWYU pragma: keep
 #include <stdio.h>
 #endif
 
-#if !defined(DBUG_OFF)
+#if !defined(NDEBUG)
 
 struct _db_stack_frame_ {
   const char *func;   /* function name of the previous stack frame       */
@@ -116,9 +116,11 @@ class AutoDebugTrace {
     const char *end = strchr(function, '(');
 
     if (end == nullptr) {
-      _db_enter_(function, strlen(function), filename, line, &m_stack_frame);
+      _db_enter_(function, static_cast<int>(strlen(function)), filename, line,
+                 &m_stack_frame);
     } else {
-      _db_enter_(function, end - function, filename, line, &m_stack_frame);
+      _db_enter_(function, static_cast<int>(end - function), filename, line,
+                 &m_stack_frame);
     }
   }
 
@@ -128,22 +130,9 @@ class AutoDebugTrace {
   _db_stack_frame_ m_stack_frame;
 };
 
-#ifdef __SUNPRO_CC
-// Disable debug tracing for Developer Studio, because we may get
-// a fatal error from ld when linking large executables.
-//   section .eh_frame%__gthread_trigger():
-//   unexpected negative integer encountered: offset 0x630
-#define DBUG_TRACE \
-  do {             \
-  } while (false)
-
-#else
-
 #define DBUG_TRACE \
   AutoDebugTrace _db_trace(DBUG_PRETTY_FUNCTION, __FILE__, __LINE__)
 
-#endif  // __SUNPRO_CC
-
 #endif
 
 #define DBUG_ENTER(a)                       \
@@ -194,7 +183,6 @@ class AutoDebugTrace {
 #define DBUG_END() _db_end_()
 #define DBUG_LOCK_FILE _db_lock_file_()
 #define DBUG_UNLOCK_FILE _db_unlock_file_()
-#define DBUG_ASSERT(A) assert(A)
 #define DBUG_EXPLAIN(buf, len) _db_explain_(0, (buf), (len))
 #define DBUG_EXPLAIN_INITIAL(buf, len) _db_explain_init_((buf), (len))
 #ifndef _WIN32
@@ -275,9 +263,6 @@ extern void _db_flush_gcov_();
 #define DBUG_END() \
   do {             \
   } while (0)
-#define DBUG_ASSERT(A) \
-  do {                 \
-  } while (0)
 #define DBUG_LOCK_FILE \
   do {                 \
   } while (0)
@@ -297,7 +282,7 @@ extern void _db_flush_gcov_();
 #endif
 
 #ifdef __cplusplus
-#if !defined(DBUG_OFF)
+#if !defined(NDEBUG)
 #include <sstream>
 #include <string>
 
@@ -323,11 +308,29 @@ extern void _db_flush_gcov_();
     }                                                  \
   } while (0)
 
-#else /* DBUG_OFF */
+#else /* NDEBUG */
 #define DBUG_LOG(keyword, v) \
   do {                       \
   } while (0)
-#endif /* DBUG_OFF */
-#endif /* __cplusplus */
+#endif /* NDEBUG */
 
+/**
+   A type-safe interface to DBUG_EXECUTE_IF, where the debug action to
+   activate when the keyword is provided is given as a callable object
+   (typically a lambda).
+
+   @note The body of the callable will be checked by the compiler even
+         in optimized mode.
+
+   @param keyword String literal which will enable this debug action.
+   @param clos    Callable object taking no arguments which will be
+                  called in debug mode if the keyword is enabled.
+ */
+template <class DBGCLOS>
+inline void dbug(const char *keyword [[maybe_unused]],
+                 DBGCLOS &&clos [[maybe_unused]]) {
+  DBUG_EXECUTE_IF(keyword, clos(););
+}
+
+#endif /* __cplusplus */
 #endif /* MY_DBUG_INCLUDED */
