@@ -494,6 +494,11 @@ int list_datasource_details(DataSource *ds)
   if (ds->server     ) printf("Server:              %s\n", ds_get_utf8attr(ds->server, &ds->server8));
   if (ds->uid        ) printf("Uid:                 %s\n", ds_get_utf8attr(ds->uid, &ds->uid8));
   if (ds->pwd        ) printf("Pwd:                 %s\n", ds_get_utf8attr(ds->pwd, &ds->pwd8));
+#if MFA_ENABLED
+  if (ds->pwd1       ) printf("Pwd1:                %s\n", ds_get_utf8attr(ds->pwd1, &ds->pwd18));
+  if (ds->pwd2       ) printf("Pwd2:                %s\n", ds_get_utf8attr(ds->pwd2, &ds->pwd28));
+  if (ds->pwd3       ) printf("Pwd3:                %s\n", ds_get_utf8attr(ds->pwd3, &ds->pwd38));
+#endif
   if (ds->database   ) printf("Database:            %s\n", ds_get_utf8attr(ds->database, &ds->database8));
   if (ds->socket     ) printf("Socket:              %s\n", ds_get_utf8attr(ds->socket, &ds->socket8));
   if (ds->initstmt   ) printf("Initial statement:   %s\n", ds_get_utf8attr(ds->initstmt, &ds->initstmt8));
@@ -509,6 +514,7 @@ int list_datasource_details(DataSource *ds)
   if (ds->port && ds->has_port) printf("Port:                %d\n", ds->port);
   if (ds->plugin_dir  ) printf("Plugin directory:    %s\n", ds_get_utf8attr(ds->plugin_dir, &ds->plugin_dir8));
   if (ds->default_auth) printf("Default Authentication Library: %s\n", ds_get_utf8attr(ds->default_auth, &ds->default_auth8));
+  if (ds->oci_config_file) printf("OCI Config File: %s\n", ds_get_utf8attr(ds->oci_config_file, &ds->oci_config_file8));
   printf("Options:\n");
   if (ds->return_matching_rows) printf("\tFOUND_ROWS\n");
   if (ds->allow_big_results) printf("\tBIG_PACKETS\n");
@@ -523,6 +529,7 @@ int list_datasource_details(DataSource *ds)
   if (ds->force_use_of_named_pipes) printf("\tNAMED_PIPE\n");
   if (ds->change_bigint_columns_to_int) printf("\tNO_BIGINT\n");
   if (ds->no_catalog) printf("\tNO_CATALOG\n");
+  if (ds->no_schema) printf("\tNO_SCHEMA\n");
   if (ds->read_options_from_mycnf) printf("\tUSE_MYCNF\n");
   if (ds->safe) printf("\tSAFE\n");
   if (ds->disable_transactions) printf("\tNO_TRANSACTIONS\n");
@@ -543,6 +550,7 @@ int list_datasource_details(DataSource *ds)
   if (ds->no_tls_1) printf("\tNO_TLS_1_0\n");
   if (ds->no_tls_1_1) printf("\tNO_TLS_1_1\n");
   if (ds->no_tls_1_2) printf("\tNO_TLS_1_2\n");
+  if (ds->no_tls_1_3) printf("\tNO_TLS_1_3\n");
   if (ds->no_ssps) printf("\tNO_SSPS\n");
   if (ds->no_information_schema) printf("\tNO_I_S\n");
   if (ds->cursor_prefetch_number) printf("\tPREFETCH=%d\n", ds->cursor_prefetch_number);
