@@ -352,16 +352,24 @@ void syncTabsData(HWND hwnd, DataSource *params)
   GET_STRING_TAB(CONNECTION_TAB, initstmt);
   GET_STRING_TAB(CONNECTION_TAB, plugin_dir);
   GET_STRING_TAB(CONNECTION_TAB, default_auth);
+  GET_STRING_TAB(CONNECTION_TAB, oci_config_file);
 
-  /* 2 - Metadata*/
+#if MFA_ENABLED
+  /* 2 - MFA*/
+  GET_STRING_TAB(MFA_TAB, pwd2);
+  GET_STRING_TAB(MFA_TAB, pwd3);
+#endif
+
+  /* 3 - Metadata*/
   GET_BOOL_TAB(METADATA_TAB, change_bigint_columns_to_int);
   GET_BOOL_TAB(METADATA_TAB, handle_binary_as_char);
   GET_BOOL_TAB(METADATA_TAB, return_table_names_for_SqlDescribeCol);
   GET_BOOL_TAB(METADATA_TAB, no_catalog);
+  GET_BOOL_TAB(METADATA_TAB, no_schema);
   GET_BOOL_TAB(METADATA_TAB, limit_column_size);
   GET_BOOL_TAB(METADATA_TAB, no_information_schema);
 
-  /* 3 - Cursors/Results */
+  /* 4 - Cursors/Results */
   GET_BOOL_TAB(CURSORS_TAB, return_matching_rows);
   GET_BOOL_TAB(CURSORS_TAB, auto_increment_null_search);
   GET_BOOL_TAB(CURSORS_TAB, dynamic_cursor);
@@ -379,10 +387,10 @@ void syncTabsData(HWND hwnd, DataSource *params)
   {
     params->cursor_prefetch_number= 0;
   }
-  /* 4 - debug*/
+  /* 5 - debug*/
   GET_BOOL_TAB(DEBUG_TAB,save_queries);
 
-  /* 5 - ssl related */
+  /* 6 - ssl related */
   GET_STRING_TAB(SSL_TAB, sslkey);
   GET_STRING_TAB(SSL_TAB, sslcert);
   GET_STRING_TAB(SSL_TAB, sslca);
@@ -395,8 +403,9 @@ void syncTabsData(HWND hwnd, DataSource *params)
   GET_BOOL_TAB(SSL_TAB, no_tls_1);
   GET_BOOL_TAB(SSL_TAB, no_tls_1_1);
   GET_BOOL_TAB(SSL_TAB, no_tls_1_2);
+  GET_BOOL_TAB(SSL_TAB, no_tls_1_3);
 
-  /* 6 - Misc*/
+  /* 7 - Misc*/
   GET_BOOL_TAB(MISC_TAB, safe);
   GET_BOOL_TAB(MISC_TAB, dont_use_set_locale);
   GET_BOOL_TAB(MISC_TAB, ignore_space_after_function_names);
@@ -437,17 +446,25 @@ void syncTabs(HWND hwnd, DataSource *params)
     SET_STRING_TAB(CONNECTION_TAB,initstmt);
     SET_STRING_TAB(CONNECTION_TAB,plugin_dir);
     SET_STRING_TAB(CONNECTION_TAB,default_auth);
+    SET_STRING_TAB(CONNECTION_TAB,oci_config_file);
   }
 
-  /* 2 - Metadata*/
+#if MFA_ENABLED
+  /* 2 - MFA*/
+  SET_STRING_TAB(MFA_TAB, pwd2);
+  SET_STRING_TAB(MFA_TAB, pwd3);
+#endif
+
+  /* 3 - Metadata*/
   SET_BOOL_TAB(METADATA_TAB, change_bigint_columns_to_int);
   SET_BOOL_TAB(METADATA_TAB, handle_binary_as_char);
   SET_BOOL_TAB(METADATA_TAB, return_table_names_for_SqlDescribeCol);
   SET_BOOL_TAB(METADATA_TAB, no_catalog);
+  SET_BOOL_TAB(METADATA_TAB, no_schema);
   SET_BOOL_TAB(METADATA_TAB, limit_column_size);
   SET_BOOL_TAB(METADATA_TAB, no_information_schema);
 
-  /* 3 - Cursors/Results */
+  /* 4 - Cursors/Results */
   SET_BOOL_TAB(CURSORS_TAB, return_matching_rows);
   SET_BOOL_TAB(CURSORS_TAB, auto_increment_null_search);
   SET_BOOL_TAB(CURSORS_TAB, dynamic_cursor);
@@ -466,10 +483,10 @@ void syncTabs(HWND hwnd, DataSource *params)
     SET_UNSIGNED_TAB(CURSORS_TAB, cursor_prefetch_number);
   }
 
-  /* 4 - debug*/
+  /* 5 - debug*/
   SET_BOOL_TAB(DEBUG_TAB,save_queries);
 
-  /* 5 - ssl related */
+  /* 6 - ssl related */
 #ifdef _WIN32
   if ( getTabCtrlTabPages(SSL_TAB-1) )
 #endif
@@ -492,17 +509,16 @@ void syncTabs(HWND hwnd, DataSource *params)
     if (params->sslmode)
       SET_COMBO_TAB(SSL_TAB, sslmode);
 
-    //SET_BOOL_TAB(SSL_TAB, sslverify);
-
     if(params->rsakey)
       SET_STRING_TAB(SSL_TAB, rsakey);
 
     SET_BOOL_TAB(SSL_TAB, no_tls_1);
     SET_BOOL_TAB(SSL_TAB, no_tls_1_1);
     SET_BOOL_TAB(SSL_TAB, no_tls_1_2);
+    SET_BOOL_TAB(SSL_TAB, no_tls_1_3);
   }
 
-  /* 6 - Misc*/
+  /* 7 - Misc*/
   SET_BOOL_TAB(MISC_TAB, safe);
   SET_BOOL_TAB(MISC_TAB, dont_use_set_locale);
   SET_BOOL_TAB(MISC_TAB, ignore_space_after_function_names);
