@@ -1,30 +1,30 @@
-// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of <MySQL Product>, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "odbctap.h"
 
@@ -1072,7 +1072,7 @@ DECLARE_TEST(t_bug52996)
   int i, len;
   SQLCHAR attrs[8192];
   SQLCHAR drv[128];
-  SQLINTEGER row_count= 0;
+  SQLLEN row_count= 0;
   DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS bug52996");
@@ -1157,28 +1157,40 @@ DECLARE_TEST(t_tls_opts)
   SQLCHAR buf[1024] = { 0 };
   SQLLEN len = 0;
 
-  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
-    NULL, NULL, NULL, "NO_TLS_1_0=1;NO_TLS_1_2=1"));
+  char *opts[] = {
+    "NO_TLS_1_0=1;",
+    "NO_TLS_1_1=1;",
+    "NO_TLS_1_2=1;",
+    "NO_TLS_1_3=1;"
+  };
 
-  /* Check the affected tows */
-  ok_sql(hstmt1, "SHOW STATUS LIKE 'Ssl_version'");
-  ok_stmt(hstmt1, SQLFetch(hstmt1));
-  ok_stmt(hstmt1, SQLGetData(hstmt1, 2, SQL_C_CHAR, buf, sizeof(buf), &len));
-  // is(len == 7);
-  // is_str(buf, "TLSv1.1", 7);
-  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  char *vals[] = {
+    "TLSv1",
+    "TLSv1.1",
+    "TLSv1.2",
+    "TLSv1.3"
+  };
 
-  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
-    NULL, NULL, NULL, "NO_TLS_1_1=1;NO_TLS_1_2=1"));
+  for (int i = 0; i < 4; ++i)
+  {
+    printf("TLS Parameters: %s\n", opts[i]);
+    is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+      NULL, NULL, NULL, opts[i]));
 
-  /* Check the affected tows */
-  ok_sql(hstmt1, "SHOW STATUS LIKE 'Ssl_version'");
-  ok_stmt(hstmt1, SQLFetch(hstmt1));
-  len = 0;
-  ok_stmt(hstmt1, SQLGetData(hstmt1, 2, SQL_C_CHAR, buf, sizeof(buf), &len));
-  // is(len == 5);
-  // is_str(buf, "TLSv1", 5);
-  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+    /* Check the affected tows */
+    ok_sql(hstmt1, "SHOW STATUS LIKE 'Ssl_version'");
+    ok_stmt(hstmt1, SQLFetch(hstmt1));
+    ok_stmt(hstmt1, SQLGetData(hstmt1, 2, SQL_C_CHAR, buf, sizeof(buf), &len));
+
+    printf("SSL Version: %s\n\n", buf);
+    /*
+      Set one NO_TLS_X option and the version applied must be
+      not the same as disabled by the option
+    */
+    is(strcmp(vals[i], buf) != 0);
+
+    free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  }
 
   return OK;
 }
@@ -1222,7 +1234,7 @@ BEGIN_TESTS
   ADD_TEST(t_nativesql)
 #ifndef NO_DRIVERMANAGER
   ADD_TEST(t_reconnect)
-  // ADD_TEST(t_bug19823) TODO: Fix
+  // ADD_TEST(t_bug19823) TODO: make sure it runs with the up-to-date UnixODBC
 #endif
   ADD_TEST(charset_utf8)
   // ADD_TEST(charset_gbk) TODO: Fix
@@ -1239,9 +1251,7 @@ BEGIN_TESTS
   ADD_TEST(t_bug32727)
   ADD_TEST(t_bug28820)
   ADD_TEST(t_bug31959)
-#ifndef USE_IODBC
   ADD_TEST(t_bug41256)
-#endif
   ADD_TEST(t_bug44971)
   ADD_TEST(t_bug48603)
   ADD_TEST(t_bug45378)
