@@ -1,30 +1,30 @@
-// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of <MySQL Product>, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "odbctap.h"
 #include "../VersionInfo.h"
@@ -85,7 +85,7 @@ DECLARE_TEST(my_table_dbs)
     SQLINTEGER nrows= 0 ;
     SQLLEN lenOrNull, rowCount= 0;
 
-    
+
     ok_sql(hstmt, "DROP DATABASE IF EXISTS my_all_db_test1");
     ok_sql(hstmt, "DROP DATABASE IF EXISTS my_all_db_test2");
     ok_sql(hstmt, "DROP DATABASE IF EXISTS my_all_db_test3");
@@ -142,7 +142,8 @@ DECLARE_TEST(my_table_dbs)
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLTables(hstmt,(SQLCHAR *)"%",1,"",0,"",0,NULL,0);
+    /* Must return non-NULL catalogs */
+    rc = SQLTables(hstmt,(SQLCHAR *)SQL_ALL_CATALOGS,1,"",0,"",0,NULL,0);
     mystmt(hstmt,rc);
 
     rc = SQLFetch(hstmt);
@@ -834,8 +835,8 @@ DECLARE_TEST(t_sqltables)
   mystmt(hstmt,r);
 
   r  = SQLTables(hstmt, (SQLCHAR *)"TEST", SQL_NTS,
-                 (SQLCHAR *)"TEST", SQL_NTS, NULL, 0,
-                 (SQLCHAR *)"TABLE", SQL_NTS);
+                        (SQLCHAR *)"", SQL_NTS, NULL, 0,
+                        (SQLCHAR *)"TABLE", SQL_NTS);
   mystmt(hstmt,r);
 
   myresult(hstmt);
@@ -851,13 +852,13 @@ DECLARE_TEST(t_sqltables)
   r = SQLFreeStmt(hstmt, SQL_CLOSE);
   mystmt(hstmt,r);
 
-  r = SQLTables(hstmt, NULL, 0, (SQLCHAR *)"%", SQL_NTS, NULL, 0, NULL, 0);
-  mystmt(hstmt,r);
+  //r = SQLTables(hstmt, NULL, 0, (SQLCHAR *)"%", SQL_NTS, NULL, 0, NULL, 0);
+  //mystmt(hstmt,r);
 
-  myresult(hstmt);
+  //myresult(hstmt);
 
-  r = SQLFreeStmt(hstmt, SQL_CLOSE);
-  mystmt(hstmt,r);
+  //r = SQLFreeStmt(hstmt, SQL_CLOSE);
+  //mystmt(hstmt,r);
 
   r = SQLTables(hstmt, "", 0, "", 0, "", 0, (SQLCHAR *)"%", SQL_NTS);
   mystmt(hstmt,r);
@@ -891,7 +892,7 @@ DECLARE_TEST(my_information_schema)
   /* We need to have istest__ as the default DB */
   is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
                                         NULL, NULL, "istest__", NULL));
-  
+
   ok_stmt(hstmt1, SQLTables(hstmt1, "istest__", SQL_NTS, "", 0, "istab%",
                             SQL_NTS, NULL, 0));
 
@@ -1044,6 +1045,10 @@ DECLARE_TEST(t_bug23031)
 {
   SQLCHAR buff[255];
 
+  // Impossible to get view and commend with NO_I_S
+  if (myoption & (1 << 30))
+    return OK;
+
   ok_sql(hstmt, "DROP VIEW IF EXISTS t_bug23031_v");
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug23031_t");
   ok_sql(hstmt, "CREATE TABLE t_bug23031_t (a INT) COMMENT 'Whee!'");
@@ -1421,11 +1426,11 @@ DECLARE_TEST(t_bug12805)
 {
   DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
   SQLCHAR     dummy[10];
-  SQLULEN     length;  
+  SQLULEN     length;
   SQLUINTEGER len2;
 
-  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, 
-                                        NULL, NULL, NULL, 
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                        NULL, NULL, NULL,
                                         "COLUMN_SIZE_S32=1"));
 
   ok_sql(hstmt1, "DROP TABLE IF EXISTS bug12805");
@@ -1492,11 +1497,14 @@ DECLARE_TEST(t_bug30770)
   is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, USE_DRIVER,
                                         NULL, NULL, "", NULL));
 
+  sprintf((char *)buff, "USE %s;", mydb);
+  ok_stmt(hstmt1, SQLExecDirect(hstmt1, buff, SQL_NTS));
+
   /* Get the info from just one table.  */
   ok_stmt(hstmt1, SQLColumns(hstmt1, NULL, SQL_NTS, NULL, SQL_NTS,
                              (SQLCHAR *)"bug30770", SQL_NTS, NULL, 0));
 
-  expect_stmt(hstmt1, SQLFetch(hstmt1), SQL_NO_DATA_FOUND);
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
 
   is_str(my_fetch_str(hstmt1, buff, 3), "bug30770", 9);
   is_str(my_fetch_str(hstmt1, buff, 4), "a", 1);
@@ -1557,16 +1565,15 @@ DECLARE_TEST(t_bug39957)
 
 
 BEGIN_TESTS
+  ADD_TEST(t_sqlprocedures)
   ADD_TEST(my_columns_null)
   ADD_TEST(my_drop_table)
-  // ADD_TEST(my_table_dbs) TODO: Fix
+  ADD_TEST(my_table_dbs)
   ADD_TEST(my_colpriv)
-  ADD_TEST(t_sqlprocedures)
   ADD_TEST(t_catalog)
   ADD_TEST(tmysql_specialcols)
   ADD_TEST(t_columns)
   ADD_TEST(t_tables_bug)
-#ifndef USE_IODBC
   ADD_TEST(t_current_catalog_unicode)
   ADD_TEST(t_current_catalog_ansi)
   ADD_TEST(t_sqltables)
@@ -1574,9 +1581,8 @@ BEGIN_TESTS
   ADD_TEST(t_bug29888)
   ADD_TEST(t_bug32989)
   ADD_TEST(t_bug12805)
-#endif
   ADD_TEST(tmysql_showkeys)
-  // ADD_TEST(my_information_schema) TODO: Fix NO_IS
+  ADD_TEST(my_information_schema)
   ADD_TEST(t_bug4518)
   ADD_TEST(empty_set)
   ADD_TEST(t_bug23031)
@@ -1587,7 +1593,7 @@ BEGIN_TESTS
   ADD_TEST(t_bug19923)
   ADD_TEST(t_bug32864)
   ADD_TEST(t_bug33298)
-  // ADD_TEST(t_bug30770) TODO: Fix NO_IS
+  ADD_TEST(t_bug30770)
   ADD_TEST(t_bug36275)
   ADD_TEST(t_bug39957)
 END_TESTS
