@@ -1,30 +1,30 @@
-// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of <MySQL Product>, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "odbctap.h"
 #include "../VersionInfo.h"
@@ -46,7 +46,7 @@ DECLARE_TEST(t_bug37621)
 /*
   Check column properties for the REMARKS column
 */
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 5, szColName, sizeof(szColName), 
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 5, szColName, sizeof(szColName),
           &iName, &iType, &uiDef, &iScale, &iNullable));
 
   is_str(szColName, "REMARKS", 8);
@@ -108,7 +108,7 @@ DECLARE_TEST(t_bug34272)
 DECLARE_TEST(t_bug49660)
 {
   SQLLEN rowsCount;
-  
+
   ok_sql(hstmt, "drop database if exists bug49660");
   ok_sql(hstmt, "drop table if exists t_bug49660");
   ok_sql(hstmt, "drop table if exists t_bug49660_r");
@@ -147,7 +147,7 @@ DECLARE_TEST(t_bug49660)
 DECLARE_TEST(t_bug51422)
 {
   SQLLEN rowsCount;
-  
+
   ok_sql(hstmt, "drop table if exists t_bug51422");
   ok_sql(hstmt, "drop table if exists t_bug51422_r");
 
@@ -170,7 +170,7 @@ DECLARE_TEST(t_bug51422)
 
 
 /*
-  Bug #36441 - SQLPrimaryKeys returns mangled strings 
+  Bug #36441 - SQLPrimaryKeys returns mangled strings
 */
 DECLARE_TEST(t_bug36441)
 {
@@ -209,8 +209,8 @@ DECLARE_TEST(t_bug36441)
   {
     ok_stmt(hstmt, SQLFetch(hstmt));
 
-    is_num(catalog_len, SQL_NULL_DATA);
-    is_num(schema_len, SQL_NULL_DATA);
+    is_str(catalog, mydb, strlen(mydb));
+    is_num(schema_len, -1);
     is_str(table, "t_bug36441_0123456789", 3);
     is_str(column, key_column_name[i], 4);
     is_num(key_seq, i+1);
@@ -268,11 +268,20 @@ DECLARE_TEST(t_bug50195)
   SQLCHAR     priv[12];
   SQLLEN      len;
 
-  (void)SQLExecDirect(hstmt, (SQLCHAR *)"DROP USER bug50195@127.0.0.1", SQL_NTS);
-  (void)SQLExecDirect(hstmt, (SQLCHAR *)"DROP USER bug50195@localhost", SQL_NTS);
+  /*
+    The patch for this issue can only work with Information_Schema.
+    Therefore skipping it for NO_IS=1 case
+  */
+  if (myoption && (1 << 30))
+    return OK;
 
-  ok_sql(hstmt, "grant all on *.* to bug50195@127.0.0.1 IDENTIFIED BY 'a'");
-  ok_sql(hstmt, "grant all on *.* to bug50195@localhost IDENTIFIED BY 'a'");
+  (void)SQLExecDirect(hstmt, (SQLCHAR *)"DROP USER `bug50195`@`127.0.0.1`", SQL_NTS);
+  (void)SQLExecDirect(hstmt, (SQLCHAR *)"DROP USER `bug50195`@`localhost`", SQL_NTS);
+
+  ok_sql(hstmt, "CREATE USER `bug50195`@`127.0.0.1` IDENTIFIED BY 'a'");
+  ok_sql(hstmt, "CREATE USER `bug50195`@`localhost` IDENTIFIED BY 'a'");
+  ok_sql(hstmt, "grant all on *.* to `bug50195`@`127.0.0.1`");
+  ok_sql(hstmt, "grant all on *.* to `bug50195`@`localhost`");
 
   ok_sql(hstmt, "revoke select on *.* from bug50195@127.0.0.1");
   ok_sql(hstmt, "revoke select on *.* from bug50195@localhost");
@@ -286,7 +295,7 @@ DECLARE_TEST(t_bug50195)
 
   ok_sql(hstmt, "FLUSH PRIVILEGES");
 
-  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, 
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
                                         "bug50195", "a", NULL, NULL));
 
   ok_stmt(hstmt1, SQLTablePrivileges(hstmt1, "mysql", SQL_NTS, 0, 0, "tables_priv", SQL_NTS));
@@ -299,14 +308,14 @@ DECLARE_TEST(t_bug50195)
     ok_stmt(hstmt1, SQLGetData(hstmt1, 6, SQL_C_CHAR, priv, sizeof(priv), &len));
     is_str(priv, expected_privs[i], len);
   }
-  
+
   expect_stmt(hstmt1, SQLFetch(hstmt1),100);
 
   free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   ok_sql(hstmt, "DROP USER bug50195@127.0.0.1");
   ok_sql(hstmt, "DROP USER bug50195@localhost");
-  
+
   return OK;
 }
 
@@ -316,7 +325,7 @@ DECLARE_TEST(t_sqlprocedurecolumns)
   SQLRETURN rc= 0;
   SQLCHAR szName[255]= {0};
 
-  typedef struct 
+  typedef struct
   {
     char *c01_procedure_cat;
     char *c02_procedure_schem;
@@ -393,7 +402,7 @@ DECLARE_TEST(t_sqlprocedurecolumns)
       64,  64,      0,  0,     SQL_NULLABLE, "", 0,  SQL_VARCHAR,  0,  64,   10, "YES"},
 
     /*cat    schem  proc_name                  col_name      col_type         data_type          type_name */
-    {"test", 0,     "procedure_columns_test1", "re_param11", SQL_PARAM_INPUT, SQL_LONGVARBINARY, "long varbinary",
+    {"test", 0,     "procedure_columns_test1", "re_param11", SQL_PARAM_INPUT, SQL_LONGVARBINARY, "mediumblob",
     /*size      buf_len   dec radix  nullable      rem def sql_data_type       sub octet     pos nullable*/
       16777215, 16777215, 0,  0,     SQL_NULLABLE, "", 0,  SQL_LONGVARBINARY,  0,  16777215, 12, "YES"},
 
@@ -433,7 +442,7 @@ DECLARE_TEST(t_sqlprocedurecolumns)
      8,      8,      0,  0,     SQL_NULLABLE, "", 0,  SQL_BINARY,     0,  8,    7, "YES"},
 
     /*cat    schem  proc_name                  col_name      col_type          data_type          type_name */
-    {"test", 0,     "procedure_columns_test2", "re_param H", SQL_PARAM_INPUT, SQL_LONGVARCHAR, "long varchar",
+    {"test", 0,     "procedure_columns_test2", "re_param H", SQL_PARAM_INPUT, SQL_LONGVARCHAR, "mediumtext",
     /*size     buf_len   dec radix  nullable      rem def sql_data_type       sub octet     pos nullable*/
      16777215, 16777215, 0,  0,     SQL_NULLABLE, "", 0,  SQL_LONGVARCHAR,  0,  16777215, 8, "YES"},
 
@@ -458,9 +467,9 @@ DECLARE_TEST(t_sqlprocedurecolumns)
      255,    255,    0,  0,     SQL_NULLABLE, "", 0,  SQL_LONGVARCHAR, 0,  255,  12, "YES"},
 
     /*cat    schem  proc_name                  col_name     col_type         data_type    type_name */
-    {"test", 0,     "procedure_columns_test2", "re_paramM", SQL_PARAM_INPUT, SQL_NUMERIC,  "numeric",
+    {"test", 0,     "procedure_columns_test2", "re_paramM", SQL_PARAM_INPUT, SQL_DECIMAL,  "decimal",
     /*size buf_len  dec radix  nullable      rem def sql_data_type sub octet pos nullable*/
-      8,   10,      2,  10,    SQL_NULLABLE, "", 0,  SQL_NUMERIC,   0,  0,   13,  "YES"},
+      8,   10,      2,  10,    SQL_NULLABLE, "", 0,  SQL_DECIMAL,   0,  0,   13,  "YES"},
 
     /*cat    schem  proc_name                  col_name       col_type         data_type     type_name */
     {"test", 0,     "procedure_columns_test3", "re_param_00", SQL_PARAM_INPUT, SQL_TYPE_TIMESTAMP, "datetime",
@@ -512,7 +521,7 @@ DECLARE_TEST(t_sqlprocedurecolumns)
     /*size buf_len  dec radix  nullable      rem def sql_data_type sub octet pos nullable*/
       7,   7,       0,  0,     SQL_NULLABLE, "", 0,  SQL_CHAR,     0,  7,    2,  "YES"},
   };
-  
+
   ok_sql(hstmt, "drop procedure if exists procedure_columns_test1");
   ok_sql(hstmt, "drop procedure if exists procedure_columns_test2");
   ok_sql(hstmt, "drop procedure if exists procedure_columns_test2_noparam");
@@ -524,19 +533,19 @@ DECLARE_TEST(t_sqlprocedurecolumns)
   ok_sql(hstmt, "create procedure procedure_columns_test1(IN re_param1 TINYINT, OUT re_param2 SMALLINT," \
                 "re_param3 MEDIUMINT, INOUT `re_param 4` INT UNSIGNED, OUT re_param5 BIGINT, re_param6 FLOAT(4,2)," \
                 "OUT re_param7 DOUBLE(5, 3), IN re_param8 DECIMAL(10,3) unSIGned, re_param9 CHAR(32)," \
-                "Out re_param10 VARCHAR(64) charset utf8, ignore_param INT, re_param11 long VARBINARY)" \
+                "Out re_param10 VARCHAR(64) charset utf8, ignore_param INT, re_param11 MEDIUMBLOB)" \
                 "begin end;"
                 );
   ok_sql(hstmt, "create procedure procedure_columns_test2(IN re_paramA bloB," \
                 "IN re_paramB LONGBLOB, inout re_paramC TinyBlob, re_paramD mediumblob, IN re_paramE varbinary(128)," \
-                "OUT re_paramF binary, re_paramG binary(8), `re_param H` LONG VARCHAR, IN re_paramI TEXT," \
+                "OUT re_paramF binary, re_paramG binary(8), `re_param H` MEDIUMTEXT, IN re_paramI TEXT," \
                 "re_paramJ mediumtext, INOUT re_paramK longtext, re_paramL tinytext, re_paramM numeric(8,2))" \
                 "begin end;"
                 );
   ok_sql(hstmt, "create procedure procedure_columns_test2_noparam()"\
                 "begin end;"
                 );
-  
+
   ok_sql(hstmt, "create procedure procedure_columns_test3(IN re_param_00 datetime,"\
                 "OUT re_param_01 date, OUT re_param_02 time, INOUT re_param_03 timestamp,"\
                 "re_param_04 year)" \
@@ -557,32 +566,33 @@ DECLARE_TEST(t_sqlprocedurecolumns)
                 );
 
   ok_stmt(hstmt, SQLProcedureColumns(hstmt, NULL, 0, NULL, 0,
-                                     "procedure_columns_test%", SQL_NTS, 
+                                     "procedure_columns_test%", SQL_NTS,
                                      "re_%", SQL_NTS));
 
   while(SQLFetch(hstmt) == SQL_SUCCESS)
   {
     SQLCHAR buff[255] = {0}, *param_cat, *param_name;
     SQLUINTEGER col_size, buf_len, octet_len;
+    printf("\nRow %d\n", (iter+1));
 
     param_cat= (SQLCHAR*)my_fetch_str(hstmt, buff, 1);
-    is_str(param_cat, data_to_check[iter].c01_procedure_cat, 
+    is_str(param_cat, data_to_check[iter].c01_procedure_cat,
            strlen(data_to_check[iter].c01_procedure_cat) + 1);
 
-    is_str(my_fetch_str(hstmt, buff, 3), 
-           data_to_check[iter].c03_procedure_name, 
+    is_str(my_fetch_str(hstmt, buff, 3),
+           data_to_check[iter].c03_procedure_name,
            strlen(data_to_check[iter].c03_procedure_name) + 1);
 
     param_name= (SQLCHAR*)my_fetch_str(hstmt, buff, 4);
-    is_str(param_name, data_to_check[iter].c04_column_name, 
+    is_str(param_name, data_to_check[iter].c04_column_name,
            strlen(data_to_check[iter].c04_column_name) + 1);
 
     is_num(my_fetch_int(hstmt, 5), data_to_check[iter].c05_column_type);
 
     is_num(my_fetch_int(hstmt, 6), data_to_check[iter].c06_data_type);
 
-    is_str(my_fetch_str(hstmt, buff, 7), 
-           data_to_check[iter].c07_type_name, 
+    is_str(my_fetch_str(hstmt, buff, 7),
+           data_to_check[iter].c07_type_name,
            strlen(data_to_check[iter].c07_type_name) + 1);
 
     col_size= my_fetch_uint(hstmt, 8);
@@ -592,7 +602,7 @@ DECLARE_TEST(t_sqlprocedurecolumns)
     is_num(buf_len, data_to_check[iter].c09_buffer_length);
 
     is_num(my_fetch_int(hstmt, 10), data_to_check[iter].c10_decimal_digits);
-    
+
     is_num(my_fetch_int(hstmt, 11), data_to_check[iter].c11_num_prec_radix);
 
     is_num(my_fetch_int(hstmt, 15), data_to_check[iter].c15_sql_data_type);
@@ -604,8 +614,8 @@ DECLARE_TEST(t_sqlprocedurecolumns)
 
     is_num(my_fetch_int(hstmt, 18), data_to_check[iter].c18_ordinal_position);
 
-    is_str(my_fetch_str(hstmt, buff, 19), 
-           data_to_check[iter].c19_is_nullable, 
+    is_str(my_fetch_str(hstmt, buff, 19),
+           data_to_check[iter].c19_is_nullable,
            strlen(data_to_check[iter].c19_is_nullable + 1));
 
     ++iter;
@@ -620,7 +630,7 @@ DECLARE_TEST(t_sqlprocedurecolumns)
   ok_sql(hstmt, "drop function if exists procedure_columns_test4_func");
   ok_sql(hstmt, "drop function if exists procedure_columns_test4_func_noparam");
   ok_sql(hstmt, "drop procedure if exists procedure_columns_test5");
-  
+
 
   return OK;
 }
@@ -637,15 +647,19 @@ DECLARE_TEST(t_bug57182)
     "  insert into simp values (id, name);"
     "END");
 
+  ok_sql(hstmt, "select SPECIFIC_NAME, GROUP_CONCAT(IF(ISNULL(PARAMETER_NAME), concat('RETURN_VALUE ', DTD_IDENTIFIER), concat(PARAMETER_MODE, ' ', PARAMETER_NAME, ' ', DTD_IDENTIFIER)) ORDER BY ORDINAL_POSITION ASC SEPARATOR ', ') PARAMS_LIST, SPECIFIC_SCHEMA, ROUTINE_TYPE FROM information_schema.parameters WHERE SPECIFIC_SCHEMA = 'test' AND SPECIFIC_NAME = 'bug57182' GROUP BY SPECIFIC_NAME, SPECIFIC_SCHEMA, ROUTINE_TYPE");
+
+  my_print_non_format_result(hstmt);
+
   ok_stmt(hstmt, SQLProcedureColumns(hstmt, "test", SQL_NTS, NULL, 0,
-    "bug57182", SQL_NTS, 
+    "bug57182", SQL_NTS,
     NULL, 0));
 
   ok_stmt(hstmt, SQLRowCount(hstmt, &nRowCount));
   is_num(2, nRowCount);
-  
+
   ok_stmt(hstmt, SQLFetch(hstmt));
-  
+
   is_str(my_fetch_str(hstmt, buff, 3), "bug57182", 9);
   is_str(my_fetch_str(hstmt, buff, 4), "id", 3);
   is_str(my_fetch_str(hstmt, buff, 7), "int", 4);
@@ -658,12 +672,12 @@ DECLARE_TEST(t_bug57182)
   is_num(my_fetch_int(hstmt, 8), 20);
 
   expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
-  
+
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   /* Almost the same thing but with column specified */
   ok_stmt(hstmt, SQLProcedureColumns(hstmt, "test", SQL_NTS, NULL, 0,
-    "bug57182", SQL_NTS, 
+    "bug57182", SQL_NTS,
     "id", SQL_NTS));
 
   ok_stmt(hstmt, SQLRowCount(hstmt, &nRowCount));
@@ -680,8 +694,8 @@ DECLARE_TEST(t_bug57182)
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   /* And testing impossible column condition - expecting to get no rows */
-  ok_stmt(hstmt, SQLProcedureColumns(hstmt, "test", SQL_NTS, NULL, 0,
-    "bug57182", SQL_NTS, 
+   ok_stmt(hstmt, SQLProcedureColumns(hstmt, "test", SQL_NTS, NULL, 0,
+    "bug57182", SQL_NTS,
     "non_existing_column%", SQL_NTS));
 
   ok_stmt(hstmt, SQLRowCount(hstmt, &nRowCount));
@@ -719,7 +733,7 @@ DECLARE_TEST(t_bug55870)
      not using I_S. SQlStatistics doesn't have I_S version, but it ma change at certain point.
      Thus let's test it on NO_I_S connection too */
 
-  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, 
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL,
                                         NULL, "", "NO_I_S=1"));
 
 
@@ -777,11 +791,11 @@ DECLARE_TEST(t_bug55870)
   sprintf(query, "revoke select (c),insert (c),update (c) on bug55870 from '%s'@'localhost'", myuid);
   ok_stmt(hstmt, SQLExecDirect(hstmt, query, SQL_NTS));
 
-  
+
   ok_sql(hstmt, "drop table if exists bug55870r");
   ok_sql(hstmt, "drop table if exists bug55870_2");
   ok_sql(hstmt, "drop table if exists bug55870");
-  
+
   free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   return OK;
@@ -834,8 +848,8 @@ DECLARE_TEST(bug12824839)
                 "(id int, vc_col varchar(32) UNIQUE, int_col int,"
                 "primary key(id,int_col))");
 
-  ok_stmt(hstmt, SQLPrepare(hstmt, "SELECT * from any_non_existing_table",
-                            SQL_NTS));
+  expect_stmt(hstmt, SQLPrepare(hstmt, "SELECT * from any_non_existing_table",
+                     SQL_NTS), SQL_ERROR);
 
   /* this will fail */
   expect_stmt(hstmt, SQLNumResultCols(hstmt, &col_count), SQL_ERROR);
@@ -879,7 +893,7 @@ DECLARE_TEST(sqlcolumns_nodbselected)
   ok_sql(hstmt, "DROP TABLE IF EXISTS sqlcolumns_nodbselected");
   ok_sql(hstmt, "CREATE TABLE sqlcolumns_nodbselected (id int)");
 
-  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, USE_DRIVER, 
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, USE_DRIVER,
                                         NULL, NULL, "", NULL));
 
   ok_con(hdbc1, SQLGetInfo(hdbc1, SQL_DATABASE_NAME,
@@ -938,7 +952,7 @@ DECLARE_TEST(t_bug14085211_part1)
 
   /* Lets check if SQLTables can get these long names  */
   ok_stmt(hstmt, SQLTables(hstmt, (SQLCHAR *)db_64_name, SQL_NTS, NULL, SQL_NTS,
-                                  (SQLCHAR *)tab_64_name, SQL_NTS, 
+                                  (SQLCHAR *)tab_64_name, SQL_NTS,
                                   "TABLE,VIEW", SQL_NTS));
 
   ok_stmt(hstmt, SQLFetch(hstmt));
@@ -947,7 +961,7 @@ DECLARE_TEST(t_bug14085211_part1)
 
   /* check the table name */
   is_str(my_fetch_str(hstmt, buff, 3), tab_64_name, 64);
-  
+
   /* only one db/table match, so nothing should be in the results */
   expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
 
@@ -955,7 +969,7 @@ DECLARE_TEST(t_bug14085211_part1)
 
   /* Lets check if SQLTables can ignore 1024-characters for table name */
   expect_stmt(hstmt, SQLTables(hstmt, (SQLCHAR *)tab_1024_name, SQL_NTS, NULL, SQL_NTS,
-                                  (SQLCHAR *)tab_1024_name, SQL_NTS, 
+                                  (SQLCHAR *)tab_1024_name, SQL_NTS,
                                   "TABLE,VIEW", SQL_NTS), SQL_ERROR);
   is_num(check_sqlstate(hstmt, "HY090"), OK);
 
@@ -970,9 +984,9 @@ DECLARE_TEST(t_bug14085211_part1)
 
 DECLARE_TEST(t_bug14085211_part2)
 {
-  /* 
+  /*
     TODO: test all catalog functions for extreme lengths of
-          database, table and column names     
+          database, table and column names
   */
   return OK;
 }
@@ -1025,7 +1039,7 @@ DECLARE_TEST(t_bug14555713)
                 end");
 
   ok_stmt(hstmt, SQLProcedureColumns(hstmt, NULL, 0, NULL, 0,
-                                     "b14555713", SQL_NTS, 
+                                     "b14555713", SQL_NTS,
                                      "p%", SQL_NTS));
 
   ok_stmt(hstmt, SQLFetch(hstmt));
@@ -1043,20 +1057,20 @@ DECLARE_TEST(t_bug14555713)
 }
 
 
-/* 
-  Bug #69448 MySQL ODBC drivers incorrectly quotes TableName when 
+/*
+  Bug #69448 MySQL ODBC drivers incorrectly quotes TableName when
   calling SQLPrimaryKeys
 */
 DECLARE_TEST(t_bug69448)
 {
- 
+
   SQLCHAR buff[255];
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS `table``69448`");
 
   ok_sql(hstmt, "CREATE TABLE `table``69448`(id int primary key)");
 
-  ok_stmt(hstmt, SQLPrimaryKeys(hstmt, NULL, SQL_NTS, NULL, SQL_NTS, 
+  ok_stmt(hstmt, SQLPrimaryKeys(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
                                "table`69448", SQL_NTS));
 
   ok_stmt(hstmt, SQLFetch(hstmt));
@@ -1065,6 +1079,9 @@ DECLARE_TEST(t_bug69448)
   is_str(my_fetch_str(hstmt, buff, 3), "table`69448", 11);
   is_str(my_fetch_str(hstmt, buff, 4), "id", 2);
 
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
   ok_sql(hstmt, "DROP TABLE IF EXISTS `table``69448`");
 
   /* free the statement cursor */
@@ -1075,7 +1092,7 @@ DECLARE_TEST(t_bug69448)
 
 /*
   Bug #69554: SQL_ATTR_MAX_ROWS applies to all result sets on the statement,
-  and not connection, which is currently there as 
+  and not connection, which is currently there as
   'set @@sql_select_limit' for whole session.
 */
 DECLARE_TEST(t_bug69554)
@@ -1101,7 +1118,7 @@ DECLARE_TEST(t_bug69554)
 
   ok_sql(hstmt, "INSERT INTO table69554a(id, id2) VALUES (1, 1), (2, 2), (3, 3)");
 
-  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_MAX_ROWS, 
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_MAX_ROWS,
                                 (SQLPOINTER)1, 0));
 
   ok_sql(hstmt, "SELECT * FROM table69554a");
@@ -1152,8 +1169,8 @@ DECLARE_TEST(t_bug69554)
 
 
 /**
-  Bug#14005343: SQLTABLES PATCH (BUG 13914518) HAS TO BE EXTENDED 
-  FOR NO_I_S CASE 
+  Bug#14005343: SQLTABLES PATCH (BUG 13914518) HAS TO BE EXTENDED
+  FOR NO_I_S CASE
   Verify database returned should not be '%' for SQL_ALL_CATALOGS input.
   Also verify total tables in all databases in case of no information schema
   and in case of information schema should match.
@@ -1170,6 +1187,7 @@ DECLARE_TEST(t_bug_14005343)
     ++nrows;
     ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, database,
                              sizeof(database), NULL));
+    printf("%s\n", database);
     /* the table catalog in the results must not be '%' */
     is(database[0] != '%');
   }
@@ -1212,7 +1230,7 @@ DECLARE_TEST(t_bug_14005343)
     /* the table catalog in the results must not be '%' */
     is(database[0] != '%');
   }
-  
+
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   ok_sql(hstmt, "SELECT TABLE_NAME, TABLE_COMMENT, TABLE_TYPE, TABLE_SCHEMA "
                 " FROM (SELECT *  FROM INFORMATION_SCHEMA.TABLES ) TABLES "
@@ -1264,28 +1282,69 @@ DECLARE_TEST(t_bug_14005343)
   return OK;
 }
 
+/*
+  SQLPROCEDURECOLUMNS FUNCTION RETURNS INCOMPLETE RESULTS
+*/
+DECLARE_TEST(t_bug32504915)
+{
+  char *create_query = malloc(32000);
+  int i, col_num;
+  int max_cols = 50;
+
+  create_query[0] = '\0';
+
+  ok_sql(hstmt, "drop procedure if exists t_bug32504915");
+  strcat(create_query, "create procedure t_bug32504915(");
+
+  for(i = 0; i < max_cols; ++i)
+  {
+    char snum[10];
+    if (i)
+      strcat(create_query, ",");
+
+    strcat(create_query, "in input_parameter_0000000");
+    snprintf(snum, sizeof(snum), "%d", i);
+    strcat(create_query, snum);
+    strcat(create_query, " varchar(45)");
+  }
+  strcat(create_query, ")begin end;");
+  ok_stmt(hstmt, SQLExecDirect(hstmt, (SQLCHAR*)create_query, SQL_NTS));
+
+  ok_stmt(hstmt, SQLProcedureColumns(hstmt, NULL, 0, NULL, 0,
+                                     "t_bug32504915", SQL_NTS,
+                                     "%", SQL_NTS));
+
+  col_num = my_print_non_format_result(hstmt);
+  is_num(50, col_num);
+
+  free(create_query);
+  return OK;
+}
+
 
 BEGIN_TESTS
+  ADD_TEST(t_bug32504915)
+  ADD_TEST(t_sqlprocedurecolumns)
+  ADD_TEST(t_bug57182)
   ADD_TEST(t_bug_14005343)
   ADD_TEST(t_bug69554)
   ADD_TEST(t_bug37621)
   ADD_TEST(t_bug34272)
   ADD_TEST(t_bug49660)
-  // ADD_TEST(t_bug51422)            TODO: Fix NO_IS
   ADD_TEST(t_bug36441)
   ADD_TEST(t_bug53235)
-  // ADD_TEST(t_bug50195)            TODO: Fix NO_IS
-  // ADD_TEST(t_sqlprocedurecolumns) TODO: Fix 
-  // ADD_TEST(t_bug57182)            TODO: Fix
+  ADD_TEST(t_bug50195)
+  // ADD_TEST(t_bug51422) Fix NO_IS when FK table is given, it should only
+  //                      return keys that are PRIMARY, not UNIQUE
   ADD_TEST(t_bug55870)
   ADD_TEST(t_bug31067)
   ADD_TEST(bug12824839)
   ADD_TEST(sqlcolumns_nodbselected)
   ADD_TEST(t_bug14085211_part1)
-  // ADD_TODO(t_bug14085211_part2) TODO: Fix
+  ADD_TODO(t_bug14085211_part2)
   ADD_TEST(t_sqlcolumns_after_select)
-  // ADD_TEST(t_bug14555713) TODO: Fix
-  // ADD_TODO(t_bug69448) TODO: Fix
+  ADD_TEST(t_bug14555713)
+  ADD_TEST(t_bug69448)
 END_TESTS
 
 myoption &= ~(1 << 30);
