@@ -35,10 +35,6 @@
 /* We don't want ansi calls to be mapped to unicode counterparts, but that does not work */
 /* #define SQL_NOUNICODEMAP 1*/
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 
 #ifdef HAVE_CONFIG_H
 # include <myconf.h>
@@ -148,6 +144,7 @@ SQLCHAR *mydsn= (SQLCHAR *)"test";
 SQLCHAR *myuid= (SQLCHAR *)"root";
 SQLCHAR *mypwd= (SQLCHAR *)"";
 SQLCHAR *mysock= NULL;
+SQLCHAR *mydns_srv= NULL;
 int      myoption= 0, myport= 0, myenable_pooling= 0;
 SQLCHAR *my_str_options= (SQLCHAR *)""; /* String for additional connection options */
 SQLCHAR *myserver= (SQLCHAR *)"localhost";
@@ -187,7 +184,7 @@ char *SKIP_REASON= NULL;
 
 typedef int (*test_func)(SQLHDBC, SQLHSTMT, SQLHENV);
 static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
-		       const char *text, const char *file, int line);
+           const char *text, const char *file, int line);
 
 /* Disable _attribute__ on non-gcc compilers. */
 #if !defined(__attribute__) && !defined(__GNUC__)
@@ -264,6 +261,8 @@ void mem_gc_init()
   gc_blk.counter= 0;
 }
 
+#define SQL_IS_SUCCESS(R) ((R == SQL_SUCCESS) || (R == SQL_SUCCESS_WITH_INFO))
+
 #define DECLARE_BASIC_HANDLES(E, C, S) SQLHENV E= NULL; \
   SQLHDBC C= NULL; \
   SQLHSTMT S= NULL
@@ -317,6 +316,8 @@ int main(int argc, char **argv) \
     myauth=  (SQLCHAR *)getenv("TEST_DEFAULTAUTH"); \
   if (getenv("TEST_PLUGINDIR")) \
     myplugindir=  (SQLCHAR *)getenv("TEST_PLUGINDIR"); \
+  if (getenv("TEST_DNS_SRV")) \
+    mydns_srv= (SQLCHAR *)getenv("TEST_DNS_SRV"); \
 \
   if (argc > 1) \
     mydsn= (SQLCHAR *)argv[1]; \
@@ -383,6 +384,11 @@ int main(int argc, char **argv) \
     return SKIP; \
   } while (0)
 
+#ifdef CPP_THROW_CLASS
+#define TEST_RETURN_FAIL throw CPP_THROW_CLASS(__FILE__, __LINE__);
+#else
+#define TEST_RETURN_FAIL return FAIL;
+#endif
 
 /**
   Execute an SQL statement and bail out if the execution does not return
@@ -398,7 +404,7 @@ do { \
              "SQLExecDirect(" #statement ", \"" query "\", SQL_NTS)",\
              __FILE__, __LINE__); \
   if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -420,7 +426,7 @@ do { \
                __FILE__, __LINE__); \
     printf("# Expected %d, but got %d in %s on line %d\n", expect, rc, \
            __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0)
 
@@ -434,10 +440,10 @@ do { \
 */
 #define ok_stmt(statement, call) \
 do { \
-  SQLRETURN rc= (call); \
-  print_diag(rc, SQL_HANDLE_STMT, (statement), #call, __FILE__, __LINE__); \
-  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
-    return FAIL; \
+  SQLRETURN _rc= (call); \
+  print_diag(_rc, SQL_HANDLE_STMT, (statement), #call, __FILE__, __LINE__); \
+  if (_rc != SQL_SUCCESS && _rc != SQL_SUCCESS_WITH_INFO) \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -453,7 +459,7 @@ do { \
   SQLRETURN rc= (call); \
   print_diag(rc, SQL_HANDLE_DESC, (desc), #call, __FILE__, __LINE__); \
   if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -474,7 +480,7 @@ do { \
     print_diag(rc, (type), (hnd), #call, __FILE__, __LINE__); \
     printf("# Expected %d, but got %d in %s on line %d\n", (expect), rc, \
            __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0)
 
@@ -504,10 +510,10 @@ do { \
 */
 #define ok_env(environ, call) \
 do { \
-  SQLRETURN rc= (call); \
-  print_diag(rc, SQL_HANDLE_ENV, (environ), #call, __FILE__, __LINE__); \
-  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
-    return FAIL; \
+  SQLRETURN _rc= (call); \
+  print_diag(_rc, SQL_HANDLE_ENV, (environ), #call, __FILE__, __LINE__); \
+  if (_rc != SQL_SUCCESS && _rc != SQL_SUCCESS_WITH_INFO) \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -520,10 +526,10 @@ do { \
 */
 #define ok_con(con, call) \
 do { \
-  SQLRETURN rc= (call); \
-  print_diag(rc, SQL_HANDLE_DBC, (con), #call, __FILE__, __LINE__); \
-  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
-    return FAIL; \
+  SQLRETURN _rc= (call); \
+  print_diag(_rc, SQL_HANDLE_DBC, (con), #call, __FILE__, __LINE__); \
+  if (_rc != SQL_SUCCESS && _rc != SQL_SUCCESS_WITH_INFO) \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -535,10 +541,10 @@ do { \
 */
 #define ok_install(call) \
 do { \
-  BOOL rc= (call); \
-  print_diag_installer(rc, #call, __FILE__, __LINE__); \
-  if (!rc) \
-    return FAIL; \
+  BOOL _rc= (call); \
+  print_diag_installer(_rc, #call, __FILE__, __LINE__); \
+  if (!_rc) \
+    TEST_RETURN_FAIL; \
 } while (0)
 
 
@@ -554,7 +560,7 @@ do { \
   if (!(a)) { \
     printf("# !(%s) in %s on line %d\n", \
            #a, __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0);
 
@@ -577,7 +583,7 @@ do { \
   if (strncmp(val_a, val_b, val_len) != 0) { \
     printf("# %s ('%*s') != '%*s' in %s on line %d\n", \
            #a, val_len, val_a, val_len, val_b, __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0);
 
@@ -591,12 +597,12 @@ do { \
 */
 #define is_wstr(a, b, c) \
 do { \
-  wchar_t *val_a= (a), *val_b= (b); \
+  SQLWCHAR *val_a= (a), *val_b= (b); \
   int val_len= (int)(c); \
-  if (memcmp(val_a, val_b, val_len * sizeof(wchar_t)) != 0) { \
+  if (memcmp(val_a, val_b, val_len * sizeof(SQLWCHAR)) != 0) { \
     printf("# %s ('%*ls') != '%*ls' in %s on line %d\n", \
            #a, val_len, val_a, val_len, val_b, __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0);
 
@@ -613,7 +619,7 @@ do { \
   if (a1 != a2) { \
     printf("# %s (%lld) != %lld in %s on line %d\n", \
            #a, a1, a2, __FILE__, __LINE__); \
-    return FAIL; \
+    TEST_RETURN_FAIL; \
   } \
 } while (0)
 
@@ -645,7 +651,7 @@ int check_sqlstate_ex(SQLHANDLE hnd, SQLSMALLINT hndtype, char *sqlstate)
   finish with SQL_SUCCESS(_WITH_INFO) result
 */
 static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
-		       const char *text, const char *file, int line)
+           const char *text, const char *file, int line)
 {
   if (rc != SQL_SUCCESS)
   {
@@ -661,7 +667,7 @@ static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
     drc= SQLGetDiagRec(htype, handle, 1, sqlstate, &native_error,
                        message, SQL_MAX_MESSAGE_LENGTH - 1, &length);
 
-    if (SQL_SUCCEEDED(drc))
+    if (SQL_IS_SUCCESS(drc))
       printf("# [%6s] %*s in %s on line %d\n",
              sqlstate, length, message, file, line);
     else
@@ -685,7 +691,7 @@ static void print_diag_installer(BOOL is_success, const char *text,
     char     message[SQL_MAX_MESSAGE_LENGTH];
     SQLRETURN   drc;
     drc= SQLInstallerError(1, &error_code, message, SQL_MAX_MESSAGE_LENGTH - 1, &length);
-    if (SQL_SUCCEEDED(drc))
+    if (SQL_IS_SUCCESS(drc))
       printf("# [%s] %s in %s on line %d\n",
              text, message, file, line);
     else
@@ -772,7 +778,7 @@ static void print_diag_installer(BOOL is_success, const char *text,
 /**
   Print resultset dashes
 */
-static int my_print_dashes(SQLHSTMT hstmt, SQLSMALLINT nCol)
+int my_print_dashes(SQLHSTMT hstmt, SQLSMALLINT nCol)
 {
     SQLLEN     disp_size, nullable;
     SQLCHAR    ColName[MAX_NAME_LEN+1];
@@ -850,12 +856,12 @@ static int my_print_data(SQLHSTMT hstmt, SQLUSMALLINT index,
 /* If we just return FAIL from my_print_non_format_result - it will
    considered as 1 row by caller */
 #define mystmt_rows(hstmt,r, row)  \
-	do { \
-	print_diag(r, SQL_HANDLE_STMT, (hstmt), "mystmt_row(hstmt,r,row)", \
-	__FILE__, __LINE__); \
-	if (!SQL_SUCCEEDED(r)) \
-	  return row; \
-	} while (0)
+  do { \
+  print_diag(r, SQL_HANDLE_STMT, (hstmt), "mystmt_row(hstmt,r,row)", \
+  __FILE__, __LINE__); \
+  if (!SQL_IS_SUCCESS(r)) \
+    return row; \
+  } while (0)
 /**
 RESULT SET
 */
@@ -891,7 +897,7 @@ int my_print_non_format_result(SQLHSTMT hstmt)
     fprintf(stdout,"\n");
 
     rc = SQLFetch(hstmt);
-    while (SQL_SUCCEEDED(rc))
+    while (SQL_IS_SUCCESS(rc))
     {
         ++nRowCount;
         for (nIndex=0; nIndex< ncol; ++nIndex)
@@ -1089,7 +1095,7 @@ wchar_t *my_fetch_wstr(SQLHSTMT hstmt, SQLWCHAR *buffer, SQLUSMALLINT icol)
   SQLLEN nLen;
 
   rc= SQLGetData(hstmt, icol, SQL_WCHAR, buffer, MAX_ROW_DATA_LEN + 1, &nLen);
-  if (!SQL_SUCCEEDED(rc))
+  if (!SQL_IS_SUCCESS(rc))
     return L"";
 
   buffer[nLen/sizeof(SQLWCHAR)]= 0;
@@ -1135,7 +1141,7 @@ int mysql_min_version(SQLHDBC hdbc, char *min_version, unsigned int length)
       major1 == major2 && (minor1 > minor2 ||
                           minor1 ==  minor2 && build1 >= build2))
     {
-		  return TRUE;
+      return TRUE;
     }
   }
 
@@ -1186,6 +1192,82 @@ int mydrvconnect(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt, SQLCHAR *connIn)
 }
 
 
+/*
+  Helper function to make the connection string.
+*/
+SQLCHAR *make_conn_str(const SQLCHAR *dsn, const SQLCHAR *uid,
+                       const SQLCHAR *pwd, const SQLCHAR *db,
+                       const SQLCHAR *options, int hide_password)
+{
+  static SQLCHAR connIn[4096]= {0};
+  SQLCHAR dsn_buf[MAX_NAME_LEN]= {0};
+  SQLCHAR socket_buf[MAX_NAME_LEN]= {0};
+  /* ";database="+ we make buffer bigger for one certain test */
+  SQLCHAR     db_buf[4096]= {0};
+  /* Should fit 8 byte + ";port=" */
+  SQLCHAR     port_buf[32]= {0};
+
+  /* We never set the custom DSN, but sometimes use DRIVER instead */
+  if (dsn == NULL)
+    snprintf((char *)dsn_buf, sizeof(dsn_buf), "DSN=%s", (char *)mydsn);
+  else if (dsn == (const SQLCHAR*)USE_DRIVER)
+    snprintf((char *)dsn_buf, sizeof(dsn_buf), "DRIVER=%s", (char *)mydriver);
+  else
+    snprintf((char *)dsn_buf, sizeof(dsn_buf), "DSN=%s", (char *)dsn);
+
+  if (uid     == NULL) uid=     myuid;
+  if (pwd     == NULL) pwd=     mypwd;
+  if (db      == NULL) db=      mydb;
+  if (options == NULL) options= my_str_options;
+
+  if (hide_password)
+    pwd = (SQLCHAR*)"*************";
+
+  snprintf((char *)connIn, sizeof(connIn), "%s;UID=%s;PWD=%s;OPTION=%d",
+          (char *)dsn_buf, (char *)uid, (char *)pwd, myoption);
+
+  if (mysock && mysock[0])
+  {
+    snprintf((char *)socket_buf, sizeof(socket_buf), ";SOCKET=%s", (char *)mysock);
+    strncat((char *)connIn, (char*)socket_buf, sizeof(connIn) - 1);
+  }
+  if (db && db[0])
+  {
+    snprintf((char *)db_buf, sizeof(db_buf), ";DATABASE=%s", (char *)db);
+    strncat((char *)connIn, (char *)db_buf, sizeof(connIn) - 1);
+  }
+  if (myport)
+  {
+    snprintf((char*)port_buf, sizeof(port_buf), ";PORT=%d", myport);
+    strncat((char *)connIn, (char*)port_buf, sizeof(connIn) - 1);
+  }
+
+  if (options != NULL && options[0] > 0)
+  {
+    strncat((char*)connIn, ";", sizeof(connIn));
+    strncat((char*)connIn, (char*)options, sizeof(connIn) - 1);
+  }
+
+#if MYSQL_VERSION_ID >= 50507
+  if (init_auth_plugin)
+  {
+    init_auth_plugin= 0; /* reset the plugin init flag */
+    if (myauth && myauth[0])
+    {
+      strncat((char *)connIn, ";DEFAULTAUTH=", sizeof(connIn));
+      strncat((char *)connIn, (char *)myauth, sizeof(connIn) - 1);
+    }
+    if (myplugindir && myplugindir[0])
+    {
+      strncat((char *)connIn, ";PLUGINDIR=", sizeof(connIn));
+      strncat((char *)connIn, (char *)myplugindir, sizeof(connIn) - 1);
+    }
+  }
+#endif
+  return connIn;
+}
+
+
 /* Helper function for tests to get (additional) connection
    If dsn, uid, pwd or options is null - they defualt to mydsn, myuid, mypwd
    and my_str_options, respectively.
@@ -1195,89 +1277,21 @@ int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
                    const SQLCHAR *options)
 {
   /* Buffers have to be large enough to contain SSL options and long names */
-  SQLCHAR     connIn[4096], connOut[4096];
-  SQLCHAR     dsn_buf[MAX_NAME_LEN]= {0}, socket_buf[MAX_NAME_LEN]= {0};
-  /* ";database="+ we make buffer bigger for one certain test */
-  SQLCHAR     db_buf[4096]= {0};
-  /* Should fit 8 byte + ";port=" */
-  SQLCHAR     port_buf[32]= {0};
+  SQLCHAR     connOut[4096];
   SQLSMALLINT len;
   SQLRETURN   rc;
   SQLCHAR     driver_name[16]; /* Should be enough for myodbc library file name */
+  SQLCHAR     *connIn;
 
-  /* We never set the custom DSN, but sometimes use DRIVER instead */
-  if (dsn == NULL)
-    sprintf((char *)dsn_buf, "DSN=%s", (char *)mydsn);
-  else if (dsn == (const SQLCHAR*)USE_DRIVER)
-    sprintf((char *)dsn_buf, "DRIVER=%s", (char *)mydriver);
-  else
-    sprintf((char *)dsn_buf, "DSN=%s", (char *)dsn);
-
-  /* We never set the custom DSN, but sometimes use DRIVER instead */
-  if (dsn == NULL)
-    sprintf((char *)dsn_buf, "DSN=%s", (char *)mydsn);
-  else if (dsn == (const SQLCHAR*)USE_DRIVER)
-    sprintf((char *)dsn_buf, "DRIVER=%s", (char *)mydriver);
-  else
-    sprintf((char *)dsn_buf, "DSN=%s", (char *)dsn);
-
-  /* Defaults */
-  if (uid     == NULL) uid=     myuid;
-  if (pwd     == NULL) pwd=     mypwd;
-  if (db      == NULL) db=      mydb;
-  if (options == NULL) options= my_str_options;
-
-  sprintf((char *)connIn, "%s;UID=%s;PWD=%s;OPTION=%d",
-          (char *)dsn_buf, (char *)uid, (char *)pwd, myoption);
-
-  if (mysock && mysock[0])
-  {
-    sprintf((char *)socket_buf, ";SOCKET=%s", (char *)mysock);
-    strcat((char *)connIn, (char*)socket_buf);
-  }
-  if (db && db[0])
-  {
-    sprintf((char *)db_buf, ";DATABASE=%s", (char *)db);
-    strcat((char *)connIn, (char *)db_buf);
-  }
-  if (myport)
-  {
-    sprintf((char*)port_buf, ";PORT=%d", myport);
-    strcat((char *)connIn, (char*)port_buf);
-  }
-
-  if (options != NULL && options[0] > 0)
-  {
-    strcat((char*)connIn, ";");
-    strcat((char*)connIn, (char*)options);
-  }
-
-#if MYSQL_VERSION_ID >= 50507
-  if (init_auth_plugin)
-  {
-    init_auth_plugin= 0; /* reset the plugin init flag */
-    if (myauth && myauth[0])
-    {
-      strcat((char *)connIn, ";DEFAULTAUTH=");
-      strcat((char *)connIn, (char *)myauth);
-    }
-    if (myplugindir && myplugindir[0])
-    {
-      strcat((char *)connIn, ";PLUGINDIR=");
-      strcat((char *)connIn, (char *)myplugindir);
-    }
-  }
-#endif
-
+  connIn = make_conn_str(dsn, uid, pwd, db, options, 0);
   rc= SQLDriverConnect(*hdbc, NULL, connIn, SQL_NTS, connOut,
                        MAX_NAME_LEN, &len, SQL_DRIVER_NOPROMPT);
 
-  if (!SQL_SUCCEEDED(rc))
+  if (!SQL_IS_SUCCESS(rc))
   {
     /* re-build and print the connection string with hidden password */
     printf("# Connection failed with the following Connection string: " \
-           "\n%s;UID=%s;PWD=*******%s%s%s;%s\n",
-           dsn_buf, uid, socket_buf, db_buf, port_buf, options);
+           "\n%s\n", (char*)make_conn_str(dsn, uid, pwd, db, options, 1));
     return rc;
   }
 
@@ -1285,7 +1299,7 @@ int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
      in fact error of turning autocommit on */
   rc= SQLSetConnectAttr(*hdbc, SQL_ATTR_AUTOCOMMIT,
                                   (SQLPOINTER)SQL_AUTOCOMMIT_ON, 0);
-  if (!SQL_SUCCEEDED(rc))
+  if (!SQL_IS_SUCCESS(rc))
   {
     return rc;
   }
@@ -1294,7 +1308,7 @@ int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
   {
     rc= SQLGetInfo(*hdbc, SQL_DRIVER_NAME, driver_name, sizeof(driver_name), NULL);
 
-    if (SQL_SUCCEEDED(rc))
+    if (SQL_IS_SUCCESS(rc))
     {
       /* ANSI driver file name contains 5a.{dll|so} */
       unicode_driver= strstr((char*)driver_name, "a.") == NULL ? 1 : 0;
@@ -1324,7 +1338,7 @@ int alloc_basic_handles_with_opt(SQLHENV *henv, SQLHDBC *hdbc,
   {
     SQLRETURN rc= SQLSetEnvAttr(*henv, SQL_ATTR_CP_MATCH, (SQLPOINTER)SQL_CP_STRICT_MATCH, 0);
 
-    if( !SQL_SUCCEEDED(rc))
+    if( !SQL_IS_SUCCESS(rc))
     {
       /* No DM case? */
       is_num(check_sqlstate_ex(*henv, SQL_HANDLE_ENV, "HYC00"), OK);
@@ -1512,6 +1526,3 @@ int using_unixodbc_version(SQLHANDLE henv, SQLCHAR *ver)
   return 0;
 }
 
-#ifdef __cplusplus
-}
-#endif
