@@ -90,6 +90,14 @@ static SQLWCHAR W_UID[]= {'U', 'I', 'D', 0};
 static SQLWCHAR W_USER[]= {'U', 'S', 'E', 'R', 0};
 static SQLWCHAR W_PWD[]= {'P', 'W', 'D', 0};
 static SQLWCHAR W_PASSWORD[]= {'P', 'A', 'S', 'S', 'W', 'O', 'R', 'D', 0};
+#if MFA_ENABLED
+static SQLWCHAR W_PWD1[]= {'P', 'W', 'D', '1', 0};
+static SQLWCHAR W_PASSWORD1[]= {'P', 'A', 'S', 'S', 'W', 'O', 'R', 'D', '1', 0};
+static SQLWCHAR W_PWD2[]= {'P', 'W', 'D', '2', 0};
+static SQLWCHAR W_PASSWORD2[]= {'P', 'A', 'S', 'S', 'W', 'O', 'R', 'D', '2', 0};
+static SQLWCHAR W_PWD3[]= {'P', 'W', 'D', '3', 0};
+static SQLWCHAR W_PASSWORD3[]= {'P', 'A', 'S', 'S', 'W', 'O', 'R', 'D', '3', 0};
+#endif
 static SQLWCHAR W_DB[]= {'D', 'B', 0};
 static SQLWCHAR W_DATABASE[]= {'D', 'A', 'T', 'A', 'B', 'A', 'S', 'E', 0};
 static SQLWCHAR W_SOCKET[]= {'S', 'O', 'C', 'K', 'E', 'T', 0};
@@ -138,6 +146,8 @@ static SQLWCHAR W_NO_BIGINT[]=
   {'N','O','_','B','I','G','I','N','T',0};
 static SQLWCHAR W_NO_CATALOG[]=
   {'N','O','_','C','A','T','A','L','O','G',0};
+static SQLWCHAR W_NO_SCHEMA[]=
+  {'N','O','_','S','C','H','E','M','A',0};
 static SQLWCHAR W_USE_MYCNF[]=
   {'U','S','E','_','M','Y','C','N','F',0};
 static SQLWCHAR W_SAFE[]=
@@ -192,6 +202,8 @@ static SQLWCHAR W_NO_TLS_1_1[] =
 { 'N', 'O', '_', 'T', 'L', 'S', '_', '1', '_', '1', 0 };
 static SQLWCHAR W_NO_TLS_1_2[] =
 { 'N', 'O', '_', 'T', 'L', 'S', '_', '1', '_', '2', 0 };
+static SQLWCHAR W_NO_TLS_1_3[] =
+{ 'N', 'O', '_', 'T', 'L', 'S', '_', '1', '_', '3', 0 };
 static SQLWCHAR W_SSLMODE[] =
 { 'S', 'S', 'L', 'M', 'O', 'D', 'E', 0 };
 static SQLWCHAR W_NO_DATE_OVERFLOW[] =
@@ -200,6 +212,8 @@ static SQLWCHAR W_ENABLE_LOCAL_INFILE[] =
 { 'E', 'N', 'A', 'B', 'L', 'E', '_', 'L', 'O', 'C', 'A', 'L', '_', 'I', 'N', 'F', 'I', 'L', 'E', 0 };
 static SQLWCHAR W_LOAD_DATA_LOCAL_DIR[] =
 { 'L', 'O', 'A', 'D', '_', 'D', 'A', 'T', 'A', '_', 'L', 'O', 'C', 'A', 'L', '_', 'D', 'I', 'R', 0 };
+static SQLWCHAR W_OCI_CONFIG_FILE[] =
+{ 'O', 'C', 'I', '_', 'C', 'O', 'N', 'F', 'I', 'G', '_', 'F', 'I', 'L', 'E', 0 };
 
 /* DS_PARAM */
 /* externally used strings */
@@ -214,7 +228,11 @@ const SQLWCHAR W_INVALID_ATTR_STR[]= {'I', 'n', 'v', 'a', 'l', 'i', 'd', ' ',
 /* List of all DSN params, used when serializing to string */
 static const
 SQLWCHAR *dsnparams[]= {W_DSN, W_DRIVER, W_DESCRIPTION, W_SERVER,
-                        W_UID, W_PWD, W_DATABASE, W_SOCKET, W_INITSTMT,
+                        W_UID, W_PWD,
+#if MFA_ENABLED
+                        W_PWD1, W_PWD2, W_PWD3,
+#endif
+                        W_DATABASE, W_SOCKET, W_INITSTMT,
                         W_PORT, W_OPTION, W_CHARSET, W_SSLKEY,
                         W_SSLCERT, W_SSLCA, W_SSLCAPATH, W_SSLCIPHER,
                         W_SSLVERIFY, W_READTIMEOUT, W_WRITETIMEOUT,
@@ -232,8 +250,9 @@ SQLWCHAR *dsnparams[]= {W_DSN, W_DRIVER, W_DESCRIPTION, W_SERVER,
                         W_CAN_HANDLE_EXP_PWD, W_ENABLE_CLEARTEXT_PLUGIN,
                         W_GET_SERVER_PUBLIC_KEY, W_ENABLE_DNS_SRV, W_MULTI_HOST,
                         W_SAVEFILE, W_RSAKEY, W_PLUGIN_DIR, W_DEFAULT_AUTH,
-                        W_NO_TLS_1, W_NO_TLS_1_1, W_NO_TLS_1_2,
-                        W_SSLMODE, W_NO_DATE_OVERFLOW, W_LOAD_DATA_LOCAL_DIR};
+                        W_NO_TLS_1, W_NO_TLS_1_1, W_NO_TLS_1_2, W_NO_TLS_1_3,
+                        W_SSLMODE, W_NO_DATE_OVERFLOW, W_LOAD_DATA_LOCAL_DIR,
+                        W_OCI_CONFIG_FILE};
 static const
 int dsnparamcnt= sizeof(dsnparams) / sizeof(SQLWCHAR *);
 /* DS_PARAM */
@@ -656,6 +675,7 @@ DataSource *ds_new()
   /* non-zero DataSource defaults here */
   ds->port = 3306;
   ds->has_port = false;
+  ds->no_schema = 1;
   /* DS_PARAM */
 
   return ds;
@@ -673,6 +693,11 @@ void ds_delete(DataSource *ds)
   x_free(ds->server);
   x_free(ds->uid);
   x_free(ds->pwd);
+#if MFA_ENABLED
+  x_free(ds->pwd1);
+  x_free(ds->pwd2);
+  x_free(ds->pwd3);
+#endif
   x_free(ds->database);
   x_free(ds->socket);
   x_free(ds->initstmt);
@@ -687,6 +712,7 @@ void ds_delete(DataSource *ds)
   x_free(ds->savefile);
   x_free(ds->plugin_dir);
   x_free(ds->default_auth);
+  x_free(ds->oci_config_file);
   x_free(ds->load_data_local_dir);
 
   x_free(ds->name8);
@@ -695,6 +721,11 @@ void ds_delete(DataSource *ds)
   x_free(ds->server8);
   x_free(ds->uid8);
   x_free(ds->pwd8);
+#if MFA_ENABLED
+  x_free(ds->pwd18);
+  x_free(ds->pwd28);
+  x_free(ds->pwd38);
+#endif
   x_free(ds->database8);
   x_free(ds->socket8);
   x_free(ds->initstmt8);
@@ -709,6 +740,7 @@ void ds_delete(DataSource *ds)
   x_free(ds->savefile8);
   x_free(ds->plugin_dir8);
   x_free(ds->default_auth8);
+  x_free(ds->oci_config_file8);
   x_free(ds->load_data_local_dir8);
 
   x_free(ds);
@@ -785,6 +817,20 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *strdest= &ds->pwd;
   else if (!sqlwcharcasecmp(W_PASSWORD, param))
     *strdest= &ds->pwd;
+#if MFA_ENABLED
+  else if (!sqlwcharcasecmp(W_PWD1, param))
+    *strdest= &ds->pwd1;
+  else if (!sqlwcharcasecmp(W_PASSWORD1, param))
+    *strdest= &ds->pwd1;
+  else if (!sqlwcharcasecmp(W_PWD2, param))
+    *strdest= &ds->pwd2;
+  else if (!sqlwcharcasecmp(W_PASSWORD2, param))
+    *strdest= &ds->pwd2;
+  else if (!sqlwcharcasecmp(W_PWD3, param))
+    *strdest= &ds->pwd3;
+  else if (!sqlwcharcasecmp(W_PASSWORD3, param))
+    *strdest= &ds->pwd3;
+#endif
   else if (!sqlwcharcasecmp(W_DB, param))
     *strdest= &ds->database;
   else if (!sqlwcharcasecmp(W_DATABASE, param))
@@ -852,6 +898,8 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *booldest= &ds->change_bigint_columns_to_int;
   else if (!sqlwcharcasecmp(W_NO_CATALOG, param))
     *booldest= &ds->no_catalog;
+  else if (!sqlwcharcasecmp(W_NO_SCHEMA, param))
+    *booldest= &ds->no_schema;
   else if (!sqlwcharcasecmp(W_USE_MYCNF, param))
     *booldest= &ds->read_options_from_mycnf;
   else if (!sqlwcharcasecmp(W_SAFE, param))
@@ -904,12 +952,16 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *booldest = &ds->no_tls_1_1;
   else if (!sqlwcharcasecmp(W_NO_TLS_1_2, param))
     *booldest = &ds->no_tls_1_2;
+  else if (!sqlwcharcasecmp(W_NO_TLS_1_3, param))
+    *booldest = &ds->no_tls_1_3;
   else if (!sqlwcharcasecmp(W_NO_DATE_OVERFLOW, param))
     *booldest = &ds->no_date_overflow;
   else if (!sqlwcharcasecmp(W_ENABLE_LOCAL_INFILE, param))
     *booldest = &ds->enable_local_infile;
   else if (!sqlwcharcasecmp(W_LOAD_DATA_LOCAL_DIR, param))
     *strdest= &ds->load_data_local_dir;
+  else if (!sqlwcharcasecmp(W_OCI_CONFIG_FILE, param))
+    *strdest= &ds->oci_config_file;
 
   /* DS_PARAM */
 }
@@ -1278,10 +1330,16 @@ int ds_add_strprop(const SQLWCHAR *name, const SQLWCHAR *propname,
  * Utility method for ds_add() to add a single integer
  * property via the installer api.
  */
-int ds_add_intprop(const SQLWCHAR *name, const SQLWCHAR *propname, int propval)
+int ds_add_intprop(const SQLWCHAR *name, const SQLWCHAR *propname, int propval,
+  bool write_zero = false)
 {
   SQLWCHAR buf[21];
   sqlwcharfromul(buf, propval);
+  if (!propval && write_zero)
+  {
+    buf[0] = (SQLWCHAR)'0';
+    buf[1] = 0;
+  }
   return ds_add_strprop(name, propname, buf);
 }
 
@@ -1344,6 +1402,11 @@ int ds_add(DataSource *ds)
   if (ds_add_strprop(ds->name, W_SERVER     , ds->server     )) goto error;
   if (ds_add_strprop(ds->name, W_UID        , ds->uid        )) goto error;
   if (ds_add_strprop(ds->name, W_PWD        , ds->pwd        )) goto error;
+#if MFA_ENABLED
+  if (ds_add_strprop(ds->name, W_PWD1       , ds->pwd1       )) goto error;
+  if (ds_add_strprop(ds->name, W_PWD2       , ds->pwd2       )) goto error;
+  if (ds_add_strprop(ds->name, W_PWD3       , ds->pwd3       )) goto error;
+#endif
   if (ds_add_strprop(ds->name, W_DATABASE   , ds->database   )) goto error;
   if (ds_add_strprop(ds->name, W_SOCKET     , ds->socket     )) goto error;
   if (ds_add_strprop(ds->name, W_INITSTMT   , ds->initstmt   )) goto error;
@@ -1378,6 +1441,7 @@ int ds_add(DataSource *ds)
   if (ds_add_intprop(ds->name, W_NAMED_PIPE, ds->force_use_of_named_pipes)) goto error;
   if (ds_add_intprop(ds->name, W_NO_BIGINT, ds->change_bigint_columns_to_int)) goto error;
   if (ds_add_intprop(ds->name, W_NO_CATALOG, ds->no_catalog)) goto error;
+  if (ds_add_intprop(ds->name, W_NO_SCHEMA, ds->no_schema, true)) goto error;
   if (ds_add_intprop(ds->name, W_USE_MYCNF, ds->read_options_from_mycnf)) goto error;
   if (ds_add_intprop(ds->name, W_SAFE, ds->safe)) goto error;
   if (ds_add_intprop(ds->name, W_NO_TRANSACTIONS, ds->disable_transactions)) goto error;
@@ -1404,9 +1468,11 @@ int ds_add(DataSource *ds)
   if (ds_add_intprop(ds->name, W_NO_TLS_1, ds->no_tls_1)) goto error;
   if (ds_add_intprop(ds->name, W_NO_TLS_1_1, ds->no_tls_1_1)) goto error;
   if (ds_add_intprop(ds->name, W_NO_TLS_1_2, ds->no_tls_1_2)) goto error;
+  if (ds_add_intprop(ds->name, W_NO_TLS_1_3, ds->no_tls_1_3)) goto error;
   if (ds_add_intprop(ds->name, W_NO_DATE_OVERFLOW, ds->no_date_overflow)) goto error;
   if (ds_add_intprop(ds->name, W_ENABLE_LOCAL_INFILE, ds->enable_local_infile)) goto error;
   if (ds_add_strprop(ds->name, W_LOAD_DATA_LOCAL_DIR, ds->load_data_local_dir)) goto error;
+  if (ds_add_strprop(ds->name, W_OCI_CONFIG_FILE, ds->oci_config_file)) goto error;
  /* DS_PARAM */
 
   rc= 0;
